@@ -23,18 +23,19 @@
  *
  * Contributor(s): 
  *
- * $Revision: 20745 $
- * $Author: rjongbloed $
- * $Date: 2008-08-28 02:20:52 +0200 (jeu 28 aoû 2008) $
+ * $Revision: 22351 $
+ * $Author: csoutheren $
+ * $Date: 2009-03-31 07:10:48 +0000 (Tue, 31 Mar 2009) $
  */
 
-#ifndef __OPAL_VIDCODEC_H
-#define __OPAL_VIDCODEC_H
+#ifndef OPAL_CODEC_VIDCODEC_H
+#define OPAL_CODEC_VIDCODEC_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <opal/transcoders.h>
 
@@ -140,17 +141,25 @@ class OpalVideoTranscoder : public OpalTranscoder
       RTP_DataFrame & output        ///<  Output data
     );
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
+
+    bool WasLastFrameIFrame() const { return lastFrameWasIFrame; }
+    void ForceIFrame()              { forceIFrame = true; }
+
+    PINDEX GetMaxEncoderOutputSize() const { return videoEncoderMaxOutputSize; }
+
   //@}
 
   protected:
     PINDEX inDataSize;
     PINDEX outDataSize;
+    PINDEX videoEncoderMaxOutputSize;
     bool   forceIFrame;
+    bool   lastFrameWasIFrame;
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     DWORD m_totalFrames;
     DWORD m_keyFrames;
 #endif
@@ -216,6 +225,6 @@ class OpalLostPicture : public OpalMediaCommand
 
 #endif // OPAL_VIDEO
 
-#endif // __OPAL_VIDCODEC_H
+#endif // OPAL_CODEC_VIDCODEC_H
 
 /////////////////////////////////////////////////////////////////////////////
