@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 22640 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
  */
 
-#ifndef __OPAL_H323CON_H
-#define __OPAL_H323CON_H
+#ifndef OPAL_H323_H323CON_H
+#define OPAL_H323_H323CON_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -40,6 +40,8 @@
 
 #include <opal/buildopts.h>
 
+#if OPAL_H323
+
 #include <opal/rtpconn.h>
 #include <opal/guid.h>
 #include <opal/buildopts.h>
@@ -227,6 +229,36 @@ class H323Connection : public OpalRTPConnection
       */
     virtual PString GetDestinationAddress();
 
+    /**Get alerting type information of an incoming call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns an empty string.
+      */
+    virtual PString GetAlertingType() const;
+
+    /**Set alerting type information for outgoing call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns false.
+      */
+    virtual bool SetAlertingType(const PString & info);
+
     /**Get the data formats this connection is capable of operating.
        This provides a list of media data format names that an
        OpalMediaStream may be created in within this connection.
@@ -285,6 +317,8 @@ class H323Connection : public OpalRTPConnection
        H323EndPoint during a clear call.
       */
     virtual void CleanUpOnCallEnd();
+
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
   //@}
 
 
@@ -1344,6 +1378,15 @@ class H323Connection : public OpalRTPConnection
                                          ///<  Parameters for channel
       RTP_QOS * rtpqos = NULL            ///<  QoS for RTP
     );
+    
+    /**Creates a new instance of an RTP channel.
+       Allows subclasses to return a custom instance
+      */
+    virtual H323_RTPChannel * CreateRTPChannel(
+      const H323Capability & capability,
+      H323Channel::Directions direction,
+      RTP_Session & rtp
+    );
 
     /**This function is called when the remote endpoint want's to create
        a new channel.
@@ -1703,12 +1746,6 @@ class H323Connection : public OpalRTPConnection
 
   /**@name Other services */
   //@{
-    /**Request a mode change to T.38 data.
-      */
-    virtual PBoolean RequestModeChangeT38(
-      const char * capabilityNames = "T.38\nT38FaxUDP"
-    );
-
     /**Get separate H.235 authentication for the connection.
        This allows an individual ARQ to override the authentical credentials
        used in H.235 based RAS for this particular connection.
@@ -1762,10 +1799,13 @@ class H323Connection : public OpalRTPConnection
     unsigned GetCallReference() const { return callReference; }
 
     /**Get the call identifier for this connection.
-      * preserved for backwards compatibility with OpenH323 only
      */
     inline const OpalGloballyUniqueID & GetCallIdentifier() const 
-    { return GetIdentifier(); }
+    { return callIdentifier; }
+
+    /**Get the protocol-specific unique identifier for this connection.
+     */
+    virtual PString GetIdentifier() const;
 
     /**Get the conference identifier for this connection.
      */
@@ -1894,26 +1934,13 @@ class H323Connection : public OpalRTPConnection
     H4507Handler&  getH4507handler(){return *h4507handler;}
 #endif
 
-   /**Open the media channels associated with an incoming call
- 
-       This function is provided to allow an OpalConnection descendant to
-       delay the opening of the media channels associated with an incoming
-       call. By default, this function is called as soon as possible after an
-       incoming connection request has been received (i.e. an INVITE or SETUP)
-       and and do all of the work associated with handling SDP or fastStart
-       eyc
-
-       By overriding this function, the media channel open is deferred, which
-       gives the other connections in the call a chance to do something before
-       the media channels are started. This allows for features beyond those
-       provided by the OnAnswer interface
-
-       Overriding this function is ok, provided the application ensures that
-       this method in this class is called.
-      */
-    virtual PBoolean OnOpenIncomingMediaChannels();
-
     virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    
+    // H.245 - compliant handling of session IDs
+    // if no internal session ID is found, zero is returned
+    unsigned GetExternalSessionID(unsigned internalSessionID) const;
+    unsigned GetExternalSessionID(unsigned internalSessionID, const OpalMediaType & mediaType);
+    unsigned GetInternalSessionID(unsigned externalSessionID, const OpalMediaType & mediaType);
 
   protected:
     /**Internal function to check if call established.
@@ -1935,6 +1962,7 @@ class H323Connection : public OpalRTPConnection
     PBoolean                 gatekeeperRouted;
     unsigned             distinctiveRing;
     unsigned             callReference;
+    OpalGloballyUniqueID callIdentifier;
     OpalGloballyUniqueID conferenceIdentifier;
 
     PString            localDestinationAddress;
@@ -1985,7 +2013,6 @@ class H323Connection : public OpalRTPConnection
     PBoolean transmitterSidePaused;
     bool     remoteTransmitPaused;
     PBoolean earlyStart;
-    PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
     PBoolean       endSessionNeeded;
     PSyncPoint endSessionReceived;
@@ -2009,6 +2036,14 @@ class H323Connection : public OpalRTPConnection
     FastStartStates        fastStartState;
     H323LogicalChannelList fastStartChannels;
     OpalMediaStreamPtr     fastStartMediaStream;
+    
+    // H.245 - compliant handling of session IDs
+    unsigned nextSessionID;
+    typedef std::map<unsigned, unsigned> InternalToExternalSessionIDMap;
+    typedef std::map<OpalMediaType, unsigned> MediaTypeToInternalSessionIDMap;
+    InternalToExternalSessionIDMap internalToExternalSessionIDMap;
+    MediaTypeToInternalSessionIDMap mediaTypeToInternalSessionIDMap;
+    PMutex sessionIDHandlingMutex;
 
 #if PTRACING
     static const char * GetConnectionStatesName(ConnectionStates s);
@@ -2047,7 +2082,9 @@ class H323Connection : public OpalRTPConnection
 PDICTIONARY(H323CallIdentityDict, PString, H323Connection);
 
 
-#endif // __OPAL_H323CON_H
+#endif // OPAL_H323
+
+#endif // OPAL_H323_H323CON_H
 
 
 /////////////////////////////////////////////////////////////////////////////
