@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22053 $
- * $Author: csoutheren $
- * $Date: 2009-02-12 07:30:50 +0000 (Thu, 12 Feb 2009) $
+ * $Revision: 22527 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-06 08:58:09 +0000 (Wed, 06 May 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -131,12 +131,12 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     /**Get a position in the list of the first entry matching the wildcard.
        The wildcard string is a string match using several special
        characters.
-       
+
        The '*' character indicates substrings, for example: "G.711*" would
        match "G.711-uLaw-64k" and "G.711-ALaw-64k".
 
-       The '@' character indicates a type of media format, Currently only
-       "@audio" and "@video" is supported.
+       The '@' character indicates a type of media format, so say "@video
+       would find the first video codec.
 
        The '!' character indicates a negative test. That is the first entry
        that does NOT match the string is returned. The string after the '!'
@@ -167,6 +167,20 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     /**Remove all the formats specified.
        Each string in the array is checked using the wildcard matching algorithm in
        FindFormat().
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "@video
+       would find the first video codec.
+
+       The '!' character indicates a negative test. That is the first entry
+       that does NOT match the string is returned. The string after the '!'
+       may contain '*' and '@' characters.
+
+       I should be noted that when the ! operator is used, codecs are removed
+       at that time and later ! operators may not have an affect. For example
+       {"!G.729","!G.728"} will NOT give you two codecs but on G.729.
       */
     void Remove(
       const PStringArray & mask
@@ -176,8 +190,16 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
        The order variable is an array of names and the list is reordered
        according to the order in that array.
 
-       Each string in the array is checked using the wildcard matching algorithm in
-       FindFormat().
+       Each string in the array is checked using the wildcard matching algorithm
+       similar to FindFormat().
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "@video
+       would sort by video codec.
+
+        The '!' character is not supported.
       */
     void Reorder(
       const PStringArray & order
@@ -591,9 +613,6 @@ class OpalMediaFormat : public PContainer
   PCONTAINERINFO(OpalMediaFormat, PContainer);
 
   public:
-    static const PString & BandwidthTIASOption();
-    static const PString & MaxPacketRateOption();
-
     /**Default constructor creates a PCM-16 media format.
       */
     OpalMediaFormat(
@@ -784,8 +803,8 @@ class OpalMediaFormat : public PContainer
     static const PString & TargetBitRateOption();
 
     /**Get the maximum frame size in bytes. If this returns zero then the
-       media format has no intrinsic maximum frame size, eg G.711 would 
-       return zero but G.723.1 whoud return 24.
+       media format has no intrinsic maximum frame size, eg a video format
+       would return zero but G.723.1 would return 24.
       */
     PINDEX GetFrameSize() const { PWaitAndSignal m(_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(MaxFrameSizeOption(), 0); }
     static const PString & MaxFrameSizeOption();
@@ -810,6 +829,14 @@ class OpalMediaFormat : public PContainer
     unsigned GetClockRate() const { PWaitAndSignal m(_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(ClockRateOption(), 1000); }
     static const PString & ClockRateOption();
 
+    /**Get the name of the OpalMediaOption indicating the protocol the format is being used on.
+      */
+    static const PString & ProtocolOption();
+
+    /**Get the name of the OpalMediaOption indicating the maximum RTP payload size.
+      */
+    static const PString & MaxTxPacketSizeOption();
+
     /**Get all of the option values of the format as a dictionary.
        Each entry is a name value pair.
       */
@@ -1024,15 +1051,15 @@ class OpalMediaFormat : public PContainer
     operator PString() const { PWaitAndSignal m(_mutex); return m_info == NULL ? "" : m_info->formatName; }
     operator const char *() const { PWaitAndSignal m(_mutex); return m_info == NULL ? "" : m_info->formatName; }
     bool operator==(const char * other) const { PWaitAndSignal m(_mutex); return m_info != NULL && m_info->formatName == other; }
-    bool operator!=(const char * other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName == other; }
+    bool operator!=(const char * other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName != other; }
     bool operator==(const PString & other) const { PWaitAndSignal m(_mutex); return m_info != NULL && m_info->formatName == other; }
-    bool operator!=(const PString & other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName == other; }
+    bool operator!=(const PString & other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName != other; }
     bool operator==(const OpalMediaFormat & other) const { PWaitAndSignal m(_mutex); return Compare(other) == EqualTo; }
     bool operator!=(const OpalMediaFormat & other) const { PWaitAndSignal m(_mutex); return Compare(other) != EqualTo; }
     friend bool operator==(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info != NULL && fmt.m_info->formatName == other; }
-    friend bool operator!=(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName == other; }
+    friend bool operator!=(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName != other; }
     friend bool operator==(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info != NULL && fmt.m_info->formatName == other; }
-    friend bool operator!=(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName == other; }
+    friend bool operator!=(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName != other; }
 
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
@@ -1137,8 +1164,7 @@ class OpalVideoFormat : public OpalMediaFormat
     static const PString & TemporalSpatialTradeOffOption();
     static const PString & TxKeyFramePeriodOption();
     static const PString & RateControlEnableOption();
-    static const PString & RateControlWindowSizeOption();
-    static const PString & RateControlMaxFramesSkipOption();
+    static const PString & RateControllerOption();
     static const PString & RateControllerBitRateScalerOption();
 };
 #endif
