@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21585 $
+ * $Revision: 22006 $
  * $Author: rjongbloed $
- * $Date: 2008-11-21 06:46:28 +0000 (Fri, 21 Nov 2008) $
+ * $Date: 2009-02-06 04:23:05 +0000 (Fri, 06 Feb 2009) $
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -122,6 +122,7 @@ public:
   virtual void OnFailed(SIP_PDU::StatusCodes);
 
   virtual PBoolean SendRequest(SIPHandler::State state);
+  virtual bool SendNotify(const PObject * /*body*/) { return false; }
 
   SIPEndPoint & GetEndPoint() const { return endpoint; }
 
@@ -222,9 +223,10 @@ public:
 private:
   void SendStatus(SIP_PDU::StatusCodes code, State state);
 
-  SIPSubscribe::Params m_parameters;
-  SIPDialogContext     m_dialog;
-  bool                 m_unconfirmed;
+  SIPSubscribe::Params     m_parameters;
+  SIPDialogContext         m_dialog;
+  bool                     m_unconfirmed;
+  SIPEventPackageHandler * m_packageHandler;
 };
 
 
@@ -238,6 +240,8 @@ public:
     const SIPEventPackage & eventPackage,
     const SIPDialogContext & dialog
   );
+  ~SIPNotifyHandler();
+
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
   virtual PBoolean SendRequest(SIPHandler::State state);
   virtual SIP_PDU::Methods GetMethod ()
@@ -246,6 +250,7 @@ public:
     { return m_eventPackage; }
 
   virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
+  virtual bool SendNotify(const PObject * body);
 
   enum Reasons {
     Deactivated,
@@ -257,9 +262,10 @@ public:
   };
 
 private:
-  SIPEventPackage  m_eventPackage;
-  SIPDialogContext m_dialog;
-  Reasons          m_reason;
+  SIPEventPackage          m_eventPackage;
+  SIPDialogContext         m_dialog;
+  Reasons                  m_reason;
+  SIPEventPackageHandler * m_packageHandler;
 };
 
 
@@ -391,8 +397,10 @@ struct SIPPresenceInfo
 
 /** Information for SIP "dialog" event package notification messages.
   */
-struct SIPDialogNotification
+struct SIPDialogNotification : public PObject
 {
+  PCLASSINFO(SIPDialogNotification, PObject);
+
   enum States {
     Terminated,
     Trying,
@@ -450,13 +458,9 @@ struct SIPDialogNotification
     Rendering m_rendering;
   } m_local, m_remote;
 
-  SIPDialogNotification(const PString & entity)
-    : m_entity(entity)
-    , m_initiator(false)
-    , m_state(Terminated)
-    , m_eventType(NoEvent)
-    , m_eventCode(0)
-  { }
+  SIPDialogNotification(const PString & entity = PString::Empty());
+
+  void PrintOn(ostream & strm) const;
 };
 
 
