@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22056 $
- * $Author: rjongbloed $
- * $Date: 2009-02-13 01:47:34 +0000 (Fri, 13 Feb 2009) $
+ * $Revision: 22553 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -44,7 +44,7 @@
 
 #if OPAL_SIP
 
-#include <ptclib/sockagg.h>
+#include <ptclib/threadpool.h>
 #include <opal/rtpep.h>
 #include <sip/sipcon.h>
 #include <sip/sippdu.h>
@@ -347,11 +347,15 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Find a connection that uses the specified token.
        This searches the endpoint for the connection that contains the token
        as provided by functions such as MakeConnection().
+
+       Note the token may be a "replaces" style value of the form:
+         "callid;to-tag=tag;from-tag=tag"
       */
     PSafePtr<SIPConnection> GetSIPConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
-    ) { return PSafePtrCast<OpalConnection, SIPConnection>(GetConnectionWithLock(token, mode)); }
+      PSafetyMode mode = PSafeReadWrite,
+      SIP_PDU::StatusCodes * errorCode = NULL
+    );
 
     virtual PBoolean IsAcceptedAddress(const SIPURL & toAddr);
 
@@ -715,7 +719,7 @@ class SIPEndPoint : public OpalRTPEndPoint
      * If no active registration is used, return the result of GetLocalURL
      * on the given transport.
      */
-    SIPURL GetContactURL(const OpalTransport &transport, const PString & userName, const PString & host);
+    SIPURL GetContactURL(const OpalTransport &transport, const SIPURL & localURI);
 
 
     /**Return the local URL for the given transport and user name.
@@ -838,32 +842,37 @@ class SIPEndPoint : public OpalRTPEndPoint
 
         void OnReceivedPDU();
 
-      private:
         SIPEndPoint & m_endpoint;
         PString       m_token;
         SIP_PDU     * m_pdu;
     };
 
-    typedef std::queue<SIP_PDU_Work *> SIP_PDUWorkQueue;
-
-    class SIP_PDU_Thread : public PThreadPoolWorkerBase
+    class PDUThreadPool : public PThreadPool<SIP_PDU_Work>
     {
       public:
-        SIP_PDU_Thread(PThreadPoolBase & _pool);
+        virtual WorkerThreadBase * CreateWorkerThread();
+    } threadPool;
+
+    typedef std::queue<SIP_PDU_Work *> SIP_PDUWorkQueue;
+
+    class SIP_PDU_Thread : public PDUThreadPool::WorkerThread
+    {
+      public:
+        SIP_PDU_Thread(PDUThreadPool & pool_);
+
+        void AddWork(SIP_PDU_Work * work);
+        void RemoveWork(SIP_PDU_Work * work);
         unsigned GetWorkSize() const;
-        void OnAddWork(SIP_PDU_Work * work);
-        void OnRemoveWork(SIP_PDU_Work *);
-        void Shutdown();
+
         void Main();
+        void Shutdown();
 
       protected:
-        PMutex mutex;
-        PSyncPoint sync;
-        SIP_PDUWorkQueue pduQueue;
+        PSyncPoint m_sync;
+        SIP_PDUWorkQueue m_pduQueue;
     };
 
-    typedef PThreadPool<SIP_PDU_Work, SIP_PDU_Thread> SIPMainThreadPool;
-    SIPMainThreadPool threadPool;
+    virtual void AddWork(SIP_PDU_Work * work);
 
     enum {
       HighPriority = 80,
@@ -878,7 +887,7 @@ class SIPEndPoint : public OpalRTPEndPoint
         virtual void OnAddInterface(const PIPSocket::InterfaceEntry & entry);
         virtual void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
 
-    protected:
+      protected:
         SIPEndPoint & m_endpoint;
     };
     InterfaceMonitor m_highPriorityMonitor;
