@@ -59,6 +59,11 @@
 #include "dyna.h"
 #include "mpi.h"
 
+#include <stdio.h>
+
+#include "tracer.h"
+
+DECLARE_TRACER
 
 extern "C" {
 #include LIBAVCODEC_HEADER
@@ -98,7 +103,8 @@ static struct StdSizes {
 
 static FFMPEGLibrary FFMPEGLibraryInstance(CODEC_ID_H263P);
 
-static void logCallbackFFMPEG (void* v, int level, const char* fmt , va_list arg) {
+static void logCallbackFFMPEG (void* v, int level, const char* fmt , va_list arg)
+{
   char buffer[512];
   int severity = 0;
   if (v) {
@@ -131,40 +137,111 @@ static char * num2str(int num)
   sprintf(buf, "%i", num);
   return strdup(buf);
 }
+
+#if TRACE_FILE
+
+static void DumpRTPPayload(Tracer & tracer, const RTPFrame & rtp, int max)
+{
+  if (max > rtp.GetPayloadSize())
+    max = rtp.GetPayloadSize();
+  unsigned char * ptr = rtp.GetPayloadPtr();
+  tracer.GetStream() << hex << setfill('0') << setprecision(2);
+  while (max-- > 0) 
+    tracer.GetStream() << (int) *ptr++ << ' ';
+  tracer.GetStream() << setfill(' ') << dec;
+}
+
+static ostream & RTPDump(Tracer & tracer, const RTPFrame & rtp)
+{
+  tracer.GetStream() << "seq=" << rtp.GetSequenceNumber()
+       << ",ts=" << rtp.GetTimestamp()
+       << ",mkr=" << rtp.GetMarker()
+       << ",pt=" << (int)rtp.GetPayloadType()
+       << ",ps=" << rtp.GetPayloadSize();
+  return tracer.GetStream();
+}
+
+static ostream & RFC2190Dump(Tracer & tracer, const RTPFrame & rtp)
+{
+  RTPDump(tracer, rtp);
+  if (rtp.GetPayloadSize() > 2) {
+    bool iFrame = false;
+    char mode;
+    unsigned char * payload = rtp.GetPayloadPtr();
+    if ((payload[0] & 0x80) == 0) {
+      mode = 'A';
+      iFrame = (payload[1] & 0x10) == 0;
+    }
+    else if ((payload[0] & 0x40) == 0) {
+      mode = 'B';
+      iFrame = (payload[4] & 0x80) == 0;
+    }
+    else {
+      mode = 'C';
+      iFrame = (payload[4] & 0x80) == 0;
+    }
+    tracer.GetStream() << "mode=" << mode << ",I=" << (iFrame ? "yes" : "no");
+  }
+  tracer.GetStream() << ",data=";
+  DumpRTPPayload(tracer, rtp, 10);
+  return tracer.GetStream();
+}
+
+static ostream & RFC2429Dump(Tracer & tracer, const RTPFrame & rtp)
+{
+  RTPDump(tracer, rtp);
+  tracer.GetStream() << ",data=";
+  DumpRTPPayload(tracer, rtp, 10);
+  return tracer.GetStream();
+}
+
+#define CODEC_TRACER_RTP(tracer, text, rtp, func) \
+tracer.Start(); tracer.GetStream() << text; func(tracer, rtp); tracer.End()
+
+#else
+
+#define CODEC_TRACER_RTP(tracer, text, rtp, func) 
+
+#endif
       
 /////////////////////////////////////////////////////////////////////////////
       
 H263_Base_EncoderContext::H263_Base_EncoderContext(const char * _prefix)
   : prefix(_prefix)
+#if TRACE_FILE
+  , tracer(_prefix, true)
+#endif
 { 
+  _inputFrameBuffer = NULL;
 }
 
 H263_Base_EncoderContext::~H263_Base_EncoderContext()
 {
+  free(_inputFrameBuffer);
 }
 
 bool H263_Base_EncoderContext::Open(CodecID codecId)
 {
-TRACE(1, prefix << "\tEncoder\tInitialising encode for RFC2190");
+  TRACE_AND_LOG(tracer, 1, "Opening encoder");
 
   if (!FFMPEGLibraryInstance.IsLoaded())
     return false;
 
   _codec = FFMPEGLibraryInstance.AvcodecFindEncoder(codecId);
   if (_codec == NULL) {
-    TRACE(1, prefix << "\tEncoder\tCodec not found for encoder");
+    TRACE_AND_LOG(tracer, 1, "Codec not found for encoder");
     return false;
   }
 
   _context = FFMPEGLibraryInstance.AvcodecAllocContext();
   if (_context == NULL) {
-    TRACE(1, prefix << "\tEncoder\tFailed to allocate context for encoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to allocate context for encoder");
     return false;
   }
 
   _inputFrame = FFMPEGLibraryInstance.AvcodecAllocFrame();
   if (_inputFrame == NULL) {
-    TRACE(1, prefix << "\tEncoder\tFailed to allocate frame for encoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to allocate frame for encoder");
     return false;
   }
 
@@ -179,15 +256,12 @@ TRACE(1, prefix << "\tEncoder\tInitialising encode for RFC2190");
 
   _context->max_b_frames = 0;
   _context->pix_fmt = PIX_FMT_YUV420P;
-  //_context->rtp_mode = 1;
-  //_context->rtp_payload_size = 500;
 
   // X-Lite does not like Custom Picture frequency clocks...
   _context->time_base.num = 100; 
   _context->time_base.den = 2997;
   _context->gop_size      = 125;
 
-//  _context->flags = 0;
   // avoid copying input/output
   _context->flags |= CODEC_FLAG_INPUT_PRESERVED; // we guarantee to preserve input for max_b_frames+1 frames
   _context->flags |= CODEC_FLAG_EMU_EDGE;        // don't draw edges
@@ -205,7 +279,7 @@ TRACE(1, prefix << "\tEncoder\tInitialising encode for RFC2190");
   SetFrameWidth(_height);
   SetFrameHeight(_width);
   SetTargetBitrate(256000);
-  SetTSTO(31);
+  SetTSTO(0);
   DisableAnnex(D);
   DisableAnnex(F);
   DisableAnnex(I);
@@ -215,7 +289,7 @@ TRACE(1, prefix << "\tEncoder\tInitialising encode for RFC2190");
 
   _frameCount = 0;
 
-  TRACE(3, prefix << "\tEncoder\tencoder created");
+  TRACE_AND_LOG(tracer, 3, "encoder created");
 
   return true;
 }
@@ -231,14 +305,17 @@ void H263_Base_EncoderContext::SetTargetBitrate (unsigned rate)
   _context->bit_rate_tolerance = rate >> 1;
   _context->rc_min_rate = 0;                   // minimum bitrate
   _context->rc_max_rate = rate;                // maximum bitrate
+  _context->rc_buffer_size = 224;
 
   /* ratecontrol qmin qmax limiting method
      0-> clipping, 1-> use a nice continous function to limit qscale wthin qmin/qmax.
   */
   
-  _context->rc_qsquish = 0;                    // limit q by clipping 
+  _context->rc_qsquish = 0;            // limit q by clipping 
   _context->rc_eq = (char*) "1";       // rate control equation
   _context->rc_buffer_size = rate * 64;
+
+  CODEC_TRACER(tracer, "target bit rate set to " << rate);
 }
 
 void H263_Base_EncoderContext::SetFrameWidth (unsigned width)
@@ -250,12 +327,14 @@ void H263_Base_EncoderContext::SetFrameWidth (unsigned width)
   _inputFrame->linesize[1] = width / 2;
   _inputFrame->linesize[2] = width / 2;
 
+  CODEC_TRACER(tracer, "frame width set to width");
 }
 
 void H263_Base_EncoderContext::SetFrameHeight (unsigned height)
 {
   _height = height;
   FFMPEGLibraryInstance.AvSetDimensions(_context, _width, _height);
+  CODEC_TRACER(tracer, "frame height set to " << height);
 }
 
 void H263_Base_EncoderContext::SetTSTO (unsigned tsto)
@@ -278,6 +357,8 @@ void H263_Base_EncoderContext::SetTSTO (unsigned tsto)
   // Lagrange multipliers - this is how the context defaults do it:
   _context->lmin = _context->qmin * FF_QP2LAMBDA;
   _context->lmax = _context->qmax * FF_QP2LAMBDA; 
+
+  CODEC_TRACER(tracer, "TSTO set to " << tsto);
 }
 
 void H263_Base_EncoderContext::EnableAnnex (Annex annex)
@@ -292,7 +373,8 @@ void H263_Base_EncoderContext::EnableAnnex (Annex annex)
     case F:
       // Annex F: Advanced Prediction Mode
       // does not work with eyeBeam
-      _context->flags |= CODEC_FLAG_OBMC; 
+      // DO NOT ENABLE THIS FLAG. FFMPEG IS NOT THREAD_SAFE WHEN THIS FLAG IS SET
+      //_context->flags |= CODEC_FLAG_OBMC; 
       break;
     case I:
       // Annex I: Advanced Intra Coding
@@ -368,13 +450,33 @@ void H263_Base_EncoderContext::DisableAnnex (Annex annex)
   }
 }
 
+#define CODEC_TRACER_FLAG(tracer, flag) \
+CODEC_TRACER(tracer, #flag " is " << ((_context->flags & flag) ? "enabled" : "disabled"));
+
 bool H263_Base_EncoderContext::OpenCodec()
 {
   if (_codec == NULL) {
-    TRACE(1, prefix << "\tEncoder\tCodec not initialized");
+    TRACE_AND_LOG(tracer, 1, "Codec not initialized");
     return false;
   }
 
+  CODEC_TRACER(tracer, "Size is " << _width << "x" << _height);
+  CODEC_TRACER(tracer, "rc_max_rate is " <<  _context->rc_max_rate);
+  CODEC_TRACER(tracer, "GOP is " << _context->gop_size);
+  CODEC_TRACER(tracer, "qmin set to " << _context->qmin);
+  CODEC_TRACER(tracer, "qmax set to " << _context->qmax);
+  CODEC_TRACER(tracer, "mb_qmin set to " << _context->mb_qmin);
+  CODEC_TRACER(tracer, "mb_qmax set to " << _context->mb_qmax);
+  CODEC_TRACER(tracer, "bit_rate set to " << _context->bit_rate);
+  CODEC_TRACER(tracer, "bit_rate_tolerance set to " <<_context->bit_rate_tolerance);
+  CODEC_TRACER(tracer, "rc_min_rate set to " << _context->rc_min_rate);
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_H263P_UMV);
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_OBMC);
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_AC_PRED);
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_H263P_SLICE_STRUCT)
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_LOOP_FILTER);
+  CODEC_TRACER_FLAG(tracer, CODEC_FLAG_H263P_AIV);
+
   return FFMPEGLibraryInstance.AvcodecOpen(_context, _codec) == 0;
 }
 
@@ -415,7 +517,7 @@ H263_RFC2190_EncoderContext::~H263_RFC2190_EncoderContext()
     FFMPEGLibraryInstance.AvcodecFree(_context);
     FFMPEGLibraryInstance.AvcodecFree(_inputFrame);
   }  
-  TRACE(3, prefix << "\tEncoder\tencoder closed");
+  TRACE_AND_LOG(tracer, 3, "encoder closed");
 }
 
 //s->avctx->rtp_callback(s->avctx, s->ptr_lastgob, current_packet_size, number_mb)
@@ -430,7 +532,7 @@ void H263_RFC2190_EncoderContext::RTPCallBack(struct AVCodecContext *avctx, void
 {
   // sometimes, FFmpeg encodes the same frame multiple times
   // we need to detect this in order to avoid duplicating the encoded data
-  if ((_data == &packetizer.buffer[0]) && (packetizer.fragments.size() != 0)) {
+  if ((_data == packetizer.m_buffer) && (packetizer.fragments.size() != 0)) {
     packetizer.fragments.resize(0);
     currentMb = 0;
     currentBytes = 0;
@@ -450,14 +552,19 @@ bool H263_RFC2190_EncoderContext::Open()
   if (!H263_Base_EncoderContext::Open(CODEC_ID_H263))
     return false;
 
+#if LIBAVCODEC_RTP_MODE
   _context->rtp_mode = 1;
+#endif
+
   _context->rtp_payload_size = 200;
   _context->rtp_callback = &rtp_callback;
   _context->opaque = (H263_RFC2190_EncoderContext *)this; // used to separate out packets from different encode threads
 
   _context->flags &= ~CODEC_FLAG_H263P_UMV;
   _context->flags &= ~CODEC_FLAG_4MV;
+#if LIBAVCODEC_RTP_MODE
   _context->flags &= ~CODEC_FLAG_H263P_AIC;
+#endif
   _context->flags &= ~CODEC_FLAG_H263P_AIV;
   _context->flags &= ~CODEC_FLAG_H263P_SLICE_STRUCT;
   
@@ -491,7 +598,7 @@ int H263_RFC2190_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
     return 0;
 
   if (_codec == NULL) {
-    TRACE(1, prefix << "\tEncoder\tCodec not initialized");
+    TRACE_AND_LOG(tracer, 1, "Encoder\tCodec not initialized");
     return 0;
   }
 
@@ -504,18 +611,29 @@ int H263_RFC2190_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
 
   // if still running out packets from previous frame, then return it
   if (packetizer.GetPacket(dstRTP, flags) != 0) {
-    dstLen = dstRTP.GetFrameLen();
+    CODEC_TRACER_RTP(tracer, "Tx frame:", dstRTP, RFC2190Dump);
+    dstLen = dstRTP.GetHeaderSize() + dstRTP.GetPayloadSize();
+    return 1;
+  }
+
+  // zero payload means do nothing
+  if (srcRTP.GetPayloadSize() == 0) {
+    TRACE_AND_LOG(tracer, 1, "Zero payload passed");
+    dstLen = dstRTP.GetHeaderSize();
+    dstRTP.SetPayloadSize(0);
+    dstRTP.SetMarker(true);
+    flags |= 1;
     return 1;
   }
 
   // make sure the source frame is legal
   if (srcRTP.GetPayloadSize() < sizeof(PluginCodec_Video_FrameHeader)) {
-    TRACE(1,prefix << "\tEncoder\tVideo grab too small, closing down video transmission thread.");
+    TRACE_AND_LOG(tracer, 1, "Video grab too small, closing down video transmission thread.");
     return 0;
   }
   PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)srcRTP.GetPayloadPtr();
   if (header->x != 0 || header->y != 0) {
-    TRACE(1, prefix << "\tEncoder\tVideo grab of partial frame unsupported, closing down video transmission thread.");
+    TRACE_AND_LOG(tracer, 1, "Video grab of partial frame unsupported, closing down video transmission thread.");
     return 0;
   }
 
@@ -524,48 +642,113 @@ int H263_RFC2190_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
       ((unsigned) _width !=  header->width) || 
       ((unsigned) _height != header->height)) {
 
-    TRACE(4,  prefix << "\tEncoder\tFirst frame received or resolution has changed - reopening codec");
+    TRACE_AND_LOG(tracer, 4, "First frame received or resolution has changed - reopening codec");
     CloseCodec();
     SetFrameWidth(header->width);
     SetFrameHeight(header->height);
     if (!OpenCodec()) {
-      TRACE(1,  prefix << "\tEncoder\tReopening codec failed");
+      TRACE_AND_LOG(tracer, 1, "Reopening codec failed");
+      return 0;
+    }
+
+    if (_inputFrameBuffer != NULL)
+      free(_inputFrameBuffer);
+#if HAVE_POSIX_MEMALIGN
+    if (posix_memalign((void **)&_inputFrameBuffer, 64, header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2)) != 0) 
+#else
+    if ((_inputFrameBuffer = (unsigned char *)malloc(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) != NULL) 
+#endif
+    {
+      TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for frame buffer");
       return 0;
     }
   }
+
+  CODEC_TRACER(tracer, "Input:seq=" << _frameCount
+                       << ",size=" << header->width << "x" << header->height
+                       << ",I=" << ((flags && forceIFrame) ? "yes" : "no"));
+
   ++_frameCount;
 
   int size = header->width * header->height;
   int frameSize = (size * 3) >> 1;
  
   // we need FF_INPUT_BUFFER_PADDING_SIZE allocated bytes after the YVU420P image for the encoder
-  memset (_inputFrameBuffer, 0 , FF_INPUT_BUFFER_PADDING_SIZE);
-  memcpy (_inputFrameBuffer + FF_INPUT_BUFFER_PADDING_SIZE, OPAL_VIDEO_FRAME_DATA_PTR(header), frameSize);
-  memset (_inputFrameBuffer + FF_INPUT_BUFFER_PADDING_SIZE + frameSize, 0 , FF_INPUT_BUFFER_PADDING_SIZE);
+  memcpy (_inputFrameBuffer, OPAL_VIDEO_FRAME_DATA_PTR(header), frameSize);
+  memset (_inputFrameBuffer + frameSize, 0 , FF_INPUT_BUFFER_PADDING_SIZE);
+  _inputFrame->data[0] = _inputFrameBuffer;
 
-  _inputFrame->data[0] = _inputFrameBuffer + FF_INPUT_BUFFER_PADDING_SIZE;
   _inputFrame->data[1] = _inputFrame->data[0] + size;
   _inputFrame->data[2] = _inputFrame->data[1] + (size / 4);
-  _inputFrame->pict_type = 0; // (flags && forceIFrame) ? FF_I_TYPE : 0;
+  _inputFrame->pict_type = (flags && forceIFrame) ? FF_I_TYPE : 0;
 
   currentMb = 0;
   currentBytes = 0;
-  packetizer.fragments.resize(0);
-  packetizer.buffer.resize(frameSize);
-  int encodedLen = FFMPEGLibraryInstance.AvcodecEncodeVideo(_context, &packetizer.buffer[0], frameSize, _inputFrame);  
 
-  packetizer.buffer.resize(encodedLen);
+  packetizer.fragments.resize(0);
+
+  size_t newOutputSize = 100000;
+
+  if (packetizer.m_buffer != NULL) {
+    if (packetizer.m_bufferSize < newOutputSize) {
+      free(packetizer.m_buffer);
+      packetizer.m_buffer = NULL;
+    }
+  }
+  if (packetizer.m_buffer == NULL) {
+    packetizer.m_bufferSize = newOutputSize;
+#if HAVE_POSIX_MEMALIGN
+    if (posix_memalign((void **)&packetizer.m_buffer, 64, packetizer.m_bufferSize) != 0) 
+#else
+    if ((packetizer.m_buffer = (unsigned char *)malloc(packetizer.m_bufferSize)) != NULL) 
+#endif
+    {
+      TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for packet buffer");
+      return 0;
+    }
+  }
+
+  //CODEC_TRACER(tracer, "Encoder called with " << frameSize << " bytes and frame type " << _inputFrame->pict_type << " at " << header->width << "x" << header->height);
+
+  int encodedLen = FFMPEGLibraryInstance.AvcodecEncodeVideo(_context, packetizer.m_buffer, packetizer.m_bufferSize, _inputFrame);  
+
+  if (encodedLen < 0) {
+    TRACE_AND_LOG(tracer, 1, "Encoder failed");
+    return 0;
+  }
+  if (encodedLen == 0) {
+    TRACE_AND_LOG(tracer, 1, "Encoder returned empty frame");
+    dstRTP.SetPayloadSize(0);
+    dstLen = dstRTP.GetHeaderSize();
+    flags |= 1;
+    return 1;
+  }
+
+  packetizer.m_bufferLen = encodedLen;
 
   // push the encoded frame through the packetizer
+#if TRACE_FILE
+  {
+    const unsigned char * p =  packetizer.m_buffer;
+    CODEC_TRACER(tracer, "Raw data: " << hex << setfill('0') << setprecision(2)
+                         << (int)p[0] << ' ' << (int)p[1] << ' ' << (int)p[2] << ' ' << (int)p[3] << ' ' << (int)p[4]
+                         << setfill(' ') << dec);
+  }
+#endif
+
   if (packetizer.Open(srcRTP.GetTimestamp(), encodedLen) < 0) {
-    TRACE(1,  prefix << "\tEncoder\tPacketizer failed");
+    TRACE_AND_LOG(tracer, 1,  "Packetizer failed");
     flags = 1;
     return 0;
   }
 
+  CODEC_TRACER(tracer, "Encoder returned " << encodedLen << " bytes as " << packetizer.fragments.size() << " frames");
+
   // return the first encoded block of data
-  if (packetizer.GetPacket(dstRTP, flags)) 
-    dstLen = dstRTP.GetFrameLen();
+  if (packetizer.GetPacket(dstRTP, flags)) {
+    CODEC_TRACER_RTP(tracer, "Tx frame:", dstRTP, RFC2190Dump);
+    dstLen = dstRTP.GetHeaderSize() + dstRTP.GetPayloadSize();
+  }
 
   return 1;
 }
@@ -592,7 +775,7 @@ H263_RFC2429_EncoderContext::~H263_RFC2429_EncoderContext()
     FFMPEGLibraryInstance.AvcodecFree(_context);
     FFMPEGLibraryInstance.AvcodecFree(_inputFrame);
   }  
-  TRACE(3, prefix << "\tEncoder\tencoder closed");
+  TRACE_AND_LOG(tracer, 3, "encoder closed");
 }
 
 
@@ -632,7 +815,7 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
     return 0;
 
   if (_codec == NULL) {
-    TRACE(1, prefix << "\tEncoder\tCodec not initialized");
+    TRACE_AND_LOG(tracer, 1, "Codec not initialized");
     return 0;
   }
 
@@ -648,17 +831,18 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
   {
     _txH263PFrame->GetRTPFrame(dstRTP, flags);
     dstLen = dstRTP.GetFrameLen();
+    CODEC_TRACER_RTP(tracer, "Tx frame:", dstRTP, RFC2429Dump);
     return 1;
   }
 
   if (srcRTP.GetPayloadSize() < sizeof(PluginCodec_Video_FrameHeader)) {
-    TRACE(1,prefix << "\tEncoder\tVideo grab too small, closing down video transmission thread.");
+    TRACE_AND_LOG(tracer, 1, "Video grab too small, closing down video transmission thread.");
     return 0;
   }
 
   PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)srcRTP.GetPayloadPtr();
   if (header->x != 0 || header->y != 0) {
-    TRACE(1, prefix << "\tEncoder\tVideo grab of partial frame unsupported, closing down video transmission thread.");
+    TRACE_AND_LOG(tracer, 1, "Video grab of partial frame unsupported, closing down video transmission thread.");
     return 0;
   }
 
@@ -667,16 +851,30 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
       ((unsigned) _width !=  header->width) || 
       ((unsigned) _height != header->height)) {
 
-    TRACE(4,  prefix << "\tEncoder\tFirst frame received or resolution has changed - reopening codec");
+    TRACE_AND_LOG(tracer, 4, "First frame received or resolution has changed - reopening codec");
     CloseCodec();
     SetFrameWidth(header->width);
     SetFrameHeight(header->height);
     if (!OpenCodec()) {
-      TRACE(1,  prefix << "\tEncoder\tReopening codec failed");
+      TRACE_AND_LOG(tracer, 1, "Reopening codec failed");
+      return 0;
+    }
+    if (_inputFrameBuffer != NULL)
+      free(_inputFrameBuffer);
+#if HAVE_POSIX_MEMALIGN
+    if (posix_memalign((void **)&_inputFrameBuffer, 64, header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2)) != 0) {
+#else
+    if ((_inputFrameBuffer = (unsigned char *)malloc(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) != NULL) {
+#endif
+      TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for frame buffer");
       return 0;
     }
   }
 
+  CODEC_TRACER(tracer, "Input:seq=" << _frameCount
+                       << ",size=" << header->width << "x" << header->height
+                       << ",I=" << ((flags && forceIFrame) ? "yes" : "no"));
+
   int size = header->width * header->height;
   int frameSize = (size * 3) >> 1;
  
@@ -696,14 +894,17 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
   _frameCount++; 
 
   if (_txH263PFrame->GetFrameSize() == 0) {
-    TRACE(1, prefix << "\tEncoder internal error - there should be outstanding packets at this point");
+    TRACE_AND_LOG(tracer, 1, "Encoder internal error - there should be outstanding packets at this point");
     return 1;
   }
 
+  CODEC_TRACER(tracer, "Encoder created " << _txH263PFrame->GetFrameSize() << " output frames");
+
   if (_txH263PFrame->HasRTPFrames())
   {
     _txH263PFrame->GetRTPFrame(dstRTP, flags);
     dstLen = dstRTP.GetFrameLen();
+    CODEC_TRACER_RTP(tracer, "Tx frame:", dstRTP, RFC2429Dump);
     return 1;
   }
   return 1;
@@ -713,29 +914,32 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
 
 H263_Base_DecoderContext::H263_Base_DecoderContext(const char * _prefix)
   : prefix(_prefix)
+#if TRACE_FILE
+  , tracer(_prefix, false)
+#endif
 {
   if (!FFMPEGLibraryInstance.IsLoaded())
     return;
 
   if ((_codec = FFMPEGLibraryInstance.AvcodecFindDecoder(CODEC_ID_H263)) == NULL) {
-    TRACE(1, prefix << "\tDecoder\tCodec not found for decoder");
+    TRACE_AND_LOG(tracer, 1, "Codec not found for decoder");
     return;
   }
 
   _context = FFMPEGLibraryInstance.AvcodecAllocContext();
   if (_context == NULL) {
-    TRACE(1, prefix << "\tDecoder\tFailed to allocate context for decoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to allocate context for decoder");
     return;
   }
 
   _outputFrame = FFMPEGLibraryInstance.AvcodecAllocFrame();
   if (_outputFrame == NULL) {
-    TRACE(1, prefix << "\tDecoder\tFailed to allocate frame for decoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to allocate frame for decoder");
     return;
   }
 
   if (!OpenCodec()) { // decoder will re-initialise context with correct frame size
-    TRACE(1, prefix << "\tDecoder\tFailed to open codec for decoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to open codec for decoder");
     return;
   }
 
@@ -748,7 +952,7 @@ H263_Base_DecoderContext::H263_Base_DecoderContext(const char * _prefix)
     _context->debug |= FF_DEBUG_MV;
   }
 
-  TRACE(4,  prefix << "\tDecoder\tH263 decoder created");
+  TRACE_AND_LOG(tracer, 4, "Decoder created");
 }
 
 H263_Base_DecoderContext::~H263_Base_DecoderContext()
@@ -764,15 +968,17 @@ H263_Base_DecoderContext::~H263_Base_DecoderContext()
 bool H263_Base_DecoderContext::OpenCodec()
 {
   if (_codec == NULL) {
-    TRACE(1, prefix << "\tDecoder\tCodec not initialized");
+    TRACE_AND_LOG(tracer, 1, "Codec not initialized");
     return 0;
   }
 
   if (FFMPEGLibraryInstance.AvcodecOpen(_context, _codec) < 0) {
-    TRACE(1, prefix << "\tDecoder\tFailed to open H.263 decoder");
+    TRACE_AND_LOG(tracer, 1, "Failed to open H.263 decoder");
     return false;
   }
 
+  TRACE_AND_LOG(tracer, 4, "Codec opened");
+
   return true;
 }
 
@@ -781,7 +987,7 @@ void H263_Base_DecoderContext::CloseCodec()
   if (_context != NULL) {
     if (_context->codec != NULL) {
       FFMPEGLibraryInstance.AvcodecClose(_context);
-      TRACE(4, prefix << "\tDecoder\tClosed H.263 decoder" );
+      TRACE_AND_LOG(tracer, 4, "Closed H.263 decoder" );
     }
   }
 }
@@ -806,11 +1012,15 @@ H263_RFC2429_DecoderContext::~H263_RFC2429_DecoderContext()
 bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen, BYTE * dst, unsigned & dstLen, unsigned int & flags)
 {
   if (!FFMPEGLibraryInstance.IsLoaded())
-    return 0;
+    return false;
+
+  TRACE_AND_LOG(tracer, 4, "Codec opened");
 
   // create RTP frame from source buffer
   RTPFrame srcRTP(src, srcLen);
 
+  CODEC_TRACER_RTP(tracer, "Tx frame:", srcRTP, RFC2429Dump);
+
   // create RTP frame from destination buffer
   RTPFrame dstRTP(dst, dstLen, 0);
   dstLen = 0;
@@ -819,7 +1029,7 @@ bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
     _rxH263PFrame->BeginNewFrame();
     flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
     _gotAGoodFrame = false;
-    return 1;
+    return true;
   }
   
   if (srcRTP.GetMarker()==0)
@@ -830,13 +1040,13 @@ bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   if (_rxH263PFrame->GetFrameSize()==0)
   {
     _rxH263PFrame->BeginNewFrame();
-    TRACE(4, prefix << "\tDecoder\tGot an empty frame - skipping");
+    TRACE_AND_LOG(tracer, 4, "Got an empty frame - skipping");
     _skippedFrameCounter++;
     return 1;
   }
 
   if (!_rxH263PFrame->hasPicHeader()) {
-    TRACE(1, prefix << "\tDecoder\tReceived frame has no picture header - dropping");
+    TRACE_AND_LOG(tracer, 1, "Received frame has no picture header - dropping");
     _rxH263PFrame->BeginNewFrame();
     flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
     _gotAGoodFrame = false;
@@ -848,7 +1058,7 @@ bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   {
     if (!_rxH263PFrame->IsIFrame())
     {
-      TRACE(1, prefix << "\tDecoder\tWaiting for an I-Frame");
+      TRACE_AND_LOG(tracer, 1, "Waiting for an I-Frame");
       _rxH263PFrame->BeginNewFrame();
       flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
       _gotAGoodFrame = false;
@@ -859,25 +1069,25 @@ bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
 
   int gotPicture = 0;
 
-  TRACE_UP(4, prefix << "\tDecoder\tDecoding " << _rxH263PFrame->GetFrameSize()  << " bytes");
+  TRACE_AND_LOG(tracer, 4, "Decoding " << _rxH263PFrame->GetFrameSize()  << " bytes");
   int bytesDecoded = FFMPEGLibraryInstance.AvcodecDecodeVideo(_context, _outputFrame, &gotPicture, _rxH263PFrame->GetFramePtr(), _rxH263PFrame->GetFrameSize());
 
   _rxH263PFrame->BeginNewFrame();
 
   if (!gotPicture) 
   {
-    TRACE(1, prefix << "\tDecoder\tDecoded "<< bytesDecoded << " bytes without getting a Picture"); 
+    TRACE_AND_LOG(tracer, 1, "Decoded "<< bytesDecoded << " bytes without getting a Picture"); 
     _skippedFrameCounter++;
     flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
     _gotAGoodFrame = false;
     return 1;
   }
 
-  TRACE_UP(4, prefix << "\tDecoder\tDecoded " << bytesDecoded << " bytes"<< ", Resolution: " << _context->width << "x" << _context->height);
+  TRACE_AND_LOG(tracer, 4, "Decoded " << bytesDecoded << " bytes"<< ", Resolution: " << _context->width << "x" << _context->height);
 
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
   if (bytesDecoded < 0) {
-    TRACE(1, prefix << "\tDecoder\tDecoded 0 bytes");
+    TRACE_AND_LOG(tracer, 1, "Decoded 0 bytes");
     flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
     _gotAGoodFrame = false;
     return 1;
@@ -885,7 +1095,7 @@ bool H263_RFC2429_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
 
   // if decoded frame size is not legal, request an I-Frame
   if (_context->width == 0 || _context->height == 0) {
-    TRACE(1, prefix << "\tDecoder\tReceived frame with invalid size");
+    TRACE_AND_LOG(tracer, 1, "Received frame with invalid size");
     flags = (_gotAGoodFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
     _gotAGoodFrame = false;
     return 1;
@@ -945,21 +1155,54 @@ H263_RFC2190_DecoderContext::~H263_RFC2190_DecoderContext()
 {
 }
 
+static bool ReturnEmptyFrame(RTPFrame & dstRTP, unsigned & dstLen, unsigned int & flags)
+{
+  flags |= PluginCodec_ReturnCoderLastFrame;
+  dstRTP.SetPayloadSize(0);
+  dstLen = 0;
+  return true;
+}
+
 bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen, BYTE * dst, unsigned & dstLen, unsigned int & flags)
 {
   if (!FFMPEGLibraryInstance.IsLoaded())
-    return 0;
+    return false;
 
   // create RTP frame from source buffer
   RTPFrame srcRTP(src, srcLen);
 
+  // create RTP frame from destination
+  RTPFrame dstRTP(dst, dstLen, 0);
+  dstRTP.SetTimestamp(srcRTP.GetTimestamp());
+
+  if (dstLen < (12 + sizeof(PluginCodec_Video_FrameHeader))) {
+    flags = 0;
+    TRACE_AND_LOG(tracer, 1, "Destination buffer " << dstLen << " insufficient for video header");
+    ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
+
+  dstLen = 0;
+
+  CODEC_TRACER_RTP(tracer, "Rx frame:", srcRTP, RFC2190Dump);
+
   // push new frame through the depacketiser
   bool requestIFrame, isIFrame;
   int code = depacketizer.SetPacket(srcRTP, requestIFrame, isIFrame);
-  if (code <= 0)
-    return true;
+  if (code <= 0) {
+    flags = 0;
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
 
-  TRACE_UP(4, prefix << "\tDecoder\tDecoding " << depacketizer.frame.size()  << " bytes");
+  if ((depacketizer.frame.size() < 3)  ||
+      (depacketizer.frame[0] != 0x00) ||
+      (depacketizer.frame[1] != 0x00) ||
+      (depacketizer.frame[2] & 0x80) != 0x80) {
+    TRACE_AND_LOG(tracer, 1, "Frame does not start with correct code");
+    flags = PluginCodec_ReturnCoderRequestIFrame;
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
+
+  TRACE_AND_LOG(tracer, 4, "Decoder called with " << depacketizer.frame.size()  << " bytes");
 
   int gotPicture = 0;
   int bytesDecoded = FFMPEGLibraryInstance.AvcodecDecodeVideo(_context, _outputFrame, &gotPicture, &depacketizer.frame[0], depacketizer.frame.size());
@@ -967,38 +1210,61 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   depacketizer.NewFrame();
 
   if (!gotPicture) {
-    TRACE(1, prefix << "\tDecoder\tDecoded "<< bytesDecoded << " bytes without getting a Picture"); 
     flags = PluginCodec_ReturnCoderRequestIFrame;
-    return 1;
+    TRACE_AND_LOG(tracer, 1, "Decoded "<< bytesDecoded << " bytes without getting a Picture"); 
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
 
-  TRACE_UP(4, prefix << "\tDecoder\tDecoded " << bytesDecoded << " bytes"<< ", Resolution: " << _context->width << "x" << _context->height);
+  TRACE_AND_LOG(tracer, 4, "Decoder processed " << bytesDecoded << " bytes, creating frame at " << _context->width << "x" << _context->height);
 
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
   if (bytesDecoded < 0) {
     flags = PluginCodec_ReturnCoderRequestIFrame;
-    return 1;
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
+
+  if (bytesDecoded == 0) {
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
 
   // if decoded frame size is not legal, request an I-Frame
-  if (_context->width == 0 || _context->height == 0) {
-    TRACE(1, prefix << "\tDecoder\tReceived frame with invalid size");
+  if ((_context->width <= 0) ||
+      (_context->height <= 0) ||
+      (_context->width > CIF4_WIDTH) ||
+      (_context->height > CIF4_HEIGHT) ||
+      (_context->height * _context->width > (CIF4_WIDTH * CIF4_HEIGHT))) {
+    TRACE_AND_LOG(tracer, 1, "Received frame with invalid size");
     flags = PluginCodec_ReturnCoderRequestIFrame;
-    return 1;
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
 
   // create RTP frame from destination buffer
-  RTPFrame dstRTP(dst, dstLen, 0);
-  dstLen = 0;
-
   int frameBytes = (_context->width * _context->height * 12) / 8;
+  if (dstRTP.GetPayloadSize() - sizeof(PluginCodec_Video_FrameHeader) < frameBytes) {
+    TRACE_AND_LOG(tracer, 1, "Destination buffer size " << dstRTP.GetPayloadSize() << " too small for frame of size " << _context->width  << "x" <<  _context->height);
+    flags = PluginCodec_ReturnCoderRequestIFrame;
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
+
   PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)dstRTP.GetPayloadPtr();
-  header->x = header->y = 0;
-  header->width = _context->width;
+  header->x      = header->y = 0;
+  header->width  = _context->width;
   header->height = _context->height;
   int size = _context->width * _context->height;
-  if (_outputFrame->data[1] == _outputFrame->data[0] + size
-      && _outputFrame->data[2] == _outputFrame->data[1] + (size >> 2)) {
+
+  if (dstRTP.GetPayloadSize() < (frameBytes + sizeof(PluginCodec_Video_FrameHeader))) {
+    flags = 0;
+    TRACE_AND_LOG(tracer, 1, "Destination buffer " << dstLen << " insufficient for decoded data size " << header->width << "x" << header->height);
+    return ReturnEmptyFrame(dstRTP, dstLen, flags);
+  }
+
+  dstRTP.SetPayloadSize(sizeof(PluginCodec_Video_FrameHeader) + frameBytes);
+  dstLen = dstRTP.GetHeaderSize() + dstRTP.GetPayloadSize();
+
+  if (
+       (_outputFrame->data[1] == (_outputFrame->data[0] + size)) &&
+       (_outputFrame->data[2] == (_outputFrame->data[1] + (size >> 2)))
+     ) {
     memcpy(OPAL_VIDEO_FRAME_DATA_PTR(header), _outputFrame->data[0], frameBytes);
   } else {
     unsigned char *dst = OPAL_VIDEO_FRAME_DATA_PTR(header);
@@ -1012,7 +1278,7 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
         memcpy(dst, src, dst_stride*h);
         dst += dst_stride*h;
       } else {
-        while (h--) {
+        while (h-- > 0) {
           memcpy(dst, src, dst_stride);
           dst += dst_stride;
           src += src_stride;
@@ -1021,12 +1287,9 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
     }
   }
 
-  dstRTP.SetPayloadSize(sizeof(PluginCodec_Video_FrameHeader) + frameBytes);
   dstRTP.SetTimestamp(srcRTP.GetTimestamp());
   dstRTP.SetMarker(true);
 
-  dstLen = dstRTP.GetFrameLen();
-
   flags = PluginCodec_ReturnCoderLastFrame |
           (isIFrame ? PluginCodec_ReturnCoderIFrame : 0) |
           (requestIFrame ? PluginCodec_ReturnCoderRequestIFrame : 0);
@@ -1118,10 +1381,11 @@ static void FindBoundingBox(const char * const * * parm,
                                              int & maxWidth,
                                              int & maxHeight,
                                              int & frameTime,
-                                             int & bitRate)
+                                             int & targetBitRate,
+                                             int & maxBitRate)
 { 
   // initialise the MPI values to disabled
-       int i;
+  int i;
   for (i = 0; i < 5; i++)
     mpi[i] = PLUGINCODEC_MPI_DISABLED;
 
@@ -1137,8 +1401,8 @@ static void FindBoundingBox(const char * const * * parm,
   int frameRate     = 10;      // 10 fps
   int origFrameTime = 900;     // 10 fps in video RTP timestamps
   int maxBR = 0;
-  int maxBitRate = 0;
-  int targetBitRate = 0;
+  maxBitRate = 0;
+  targetBitRate = 0;
 
   // extract the MPI values set in the custom options, and find the min/max of them
   frameTime = 0;
@@ -1236,13 +1500,19 @@ static void FindBoundingBox(const char * const * * parm,
   }
 
   // find an appropriate max bit rate
-  bitRate = 0;
-  if (maxBR == 0)
-    bitRate = maxBitRate;
-  else if (maxBitRate == 0)
-    bitRate = maxBR * 100;
-  else
-    bitRate = PMIN(maxBR * 100, maxBitRate);
+  if (maxBitRate == 0) {
+    if (maxBR != 0)
+      maxBitRate = maxBR * 100;
+    else if (targetBitRate != 0)
+      maxBitRate = targetBitRate;
+    else
+      maxBitRate = 327000;
+  }
+  else if (maxBR > 0)
+    maxBitRate = PMIN(maxBR * 100, maxBitRate);
+
+  if (targetBitRate == 0)
+    targetBitRate = 327000;
 }
 
 static int to_normalised_options(const struct PluginCodec_Definition *, void *, const char *, void * parm, unsigned * parmLen)
@@ -1252,8 +1522,8 @@ static int to_normalised_options(const struct PluginCodec_Definition *, void *,
 
   // find bounding box enclosing all MPI values
   int mpi[5];
-  int minWidth, minHeight, maxHeight, maxWidth, frameTime, bitRate;
-  FindBoundingBox((const char * const * *)parm, mpi, minWidth, minHeight, maxWidth, maxHeight, frameTime, bitRate);
+  int minWidth, minHeight, maxHeight, maxWidth, frameTime, targetBitRate, maxBitRate;
+  FindBoundingBox((const char * const * *)parm, mpi, minWidth, minHeight, maxWidth, maxHeight, frameTime, targetBitRate, maxBitRate);
 
   char ** options = (char **)calloc(16+(5*2)+2, sizeof(char *));
   *(char ***)parm = options;
@@ -1271,11 +1541,11 @@ static int to_normalised_options(const struct PluginCodec_Definition *, void *,
   options[ 8] = strdup(PLUGINCODEC_OPTION_FRAME_TIME);
   options[ 9] = num2str(frameTime);
   options[10] = strdup(PLUGINCODEC_OPTION_MAX_BIT_RATE);
-  options[11] = num2str(bitRate);
+  options[11] = num2str(maxBitRate);
   options[12] = strdup(PLUGINCODEC_OPTION_TARGET_BIT_RATE);
-  options[13] = num2str(bitRate);
+  options[13] = num2str(targetBitRate);
   options[14] = strdup("MaxBR");
-  options[15] = num2str((bitRate+50)/100);
+  options[15] = num2str((maxBitRate+50)/100);
   for (int i = 0; i < 5; i++) {
     options[16+i*2] = strdup(StandardVideoSizes[i].optionName);
     options[16+i*2+1] = num2str(mpi[i]);
@@ -1291,8 +1561,8 @@ static int to_customised_options(const struct PluginCodec_Definition *, void *,
 
   // find bounding box enclosing all MPI values
   int mpi[5];
-  int minWidth, minHeight, maxHeight, maxWidth, frameTime, bitRate;
-  FindBoundingBox((const char * const * *)parm, mpi, minWidth, minHeight, maxWidth, maxHeight, frameTime, bitRate);
+  int minWidth, minHeight, maxHeight, maxWidth, frameTime, targetBitRate, maxBitRate;
+  FindBoundingBox((const char * const * *)parm, mpi, minWidth, minHeight, maxWidth, maxHeight, frameTime, targetBitRate, maxBitRate);
 
   char ** options = (char **)calloc(14+5*2+2, sizeof(char *));
   *(char ***)parm = options;
@@ -1308,11 +1578,11 @@ static int to_customised_options(const struct PluginCodec_Definition *, void *,
   options[ 6] = strdup(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);
   options[ 7] = num2str(maxHeight);
   options[ 8] = strdup(PLUGINCODEC_OPTION_MAX_BIT_RATE);
-  options[ 9] = num2str(bitRate);
+  options[ 9] = num2str(maxBitRate);
   options[10] = strdup(PLUGINCODEC_OPTION_TARGET_BIT_RATE);
-  options[11] = num2str(bitRate);
+  options[11] = num2str(targetBitRate);
   options[12] = strdup("MaxBR");
-  options[13] = num2str((bitRate+50)/100);
+  options[13] = num2str((maxBitRate+50)/100);
   for (int i = 0; i < 5; i++) {
     options[14+i*2] = strdup(StandardVideoSizes[i].optionName);
     options[14+i*2+1] = num2str(mpi[i]);
@@ -1416,7 +1686,7 @@ static int codec_decoder(const struct PluginCodec_Definition *,
                                    unsigned int * flag)
 {
   H263_Base_DecoderContext * context = (H263_Base_DecoderContext *)_context;
-  return context->DecodeFrames((const BYTE *)from, *fromLen, (BYTE *)to, *toLen, *flag);
+  return context->DecodeFrames((const BYTE *)from, *fromLen, (BYTE *)to, *toLen, *flag) ? 1 : 0;
 }
 
 static int decoder_get_output_data_size(const PluginCodec_Definition * codec, void *, const char *, void *, unsigned *)
