@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21520 $
+ * $Revision: 22253 $
  * $Author: rjongbloed $
- * $Date: 2008-11-14 03:26:33 +0000 (Fri, 14 Nov 2008) $
+ * $Date: 2009-03-24 00:37:48 +0000 (Tue, 24 Mar 2009) $
  */
 
 #ifndef _IvrOPAL_MAIN_H
@@ -41,6 +41,18 @@ class MyManager : public OpalManager
 };
 
 
+class MySIPEndPoint : public SIPEndPoint
+{
+    PCLASSINFO(MySIPEndPoint, SIPEndPoint)
+
+  public:
+    MySIPEndPoint(OpalManager & manager) : SIPEndPoint(manager) { }
+    void OnRegistrationStatus(const RegistrationStatus & status);
+
+    PSyncPoint m_completed;
+};
+
+
 class IvrOPAL : public PProcess
 {
     PCLASSINFO(IvrOPAL, PProcess)
