@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20832 $
+ * $Revision: 21912 $
  * $Author: rjongbloed $
- * $Date: 2008-09-03 03:00:52 +0000 (Wed, 03 Sep 2008) $
+ * $Date: 2009-01-10 02:47:43 +0000 (Sat, 10 Jan 2009) $
  */
 
 #pragma once
@@ -61,6 +61,7 @@ protected:
   bool m_speakerphone;
   CStringA m_currentAOR;
   CStringA m_currentHost;
+  HANDLE m_hPowerRequirement;
 
   // Generated message map functions
   virtual BOOL OnInitDialog();
@@ -73,6 +74,7 @@ protected:
   void SetCallButton(bool enabled, UINT strId = 0);
   void AddRecentCall(const CString & uri);
   void HandleMessage(OpalMessage & message);
+  void EnableFullPower();
 
 public:
   afx_msg void OnTimer(UINT_PTR nIDEvent);
