@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22153 $
+ * $Revision: 22599 $
  * $Author: rjongbloed $
- * $Date: 2009-03-04 06:13:21 +0000 (Wed, 04 Mar 2009) $
+ * $Date: 2009-05-13 04:47:49 +0000 (Wed, 13 May 2009) $
  */
 
 #ifdef __GNUC__
@@ -980,13 +980,14 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
         return false;
       }
 
-      if ((flags & PluginCodec_ReturnCoderIFrame) != 0)
-        lastFrameWasIFrame = true;
+      lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
 
-      if (toLen > 0) {
+      if ((toLen >= RTP_DataFrame::MinHeaderSize) && ((PINDEX)toLen >= dst->GetHeaderSize())) {
         dst->SetPayloadSize(toLen - dst->GetHeaderSize());
         dstList.Append(dst);
       }
+      else
+        delete dst;
 
     } while ((flags & PluginCodec_ReturnCoderLastFrame) == 0);
     PTRACE(5, "OpalPlugin\tEncoded video frame into " << dstList.GetSize() << " packets.");
@@ -1001,12 +1002,12 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
     if (!lastFrameWasIFrame)
       m_consecutiveIntraFrames = 0;
     else if (forceIFrame)
-      PTRACE(3, "OpalPlugin\tSending I-Frame in response to videoFastUpdate");
+      PTRACE(3, "OpalPlugin\tEncoder sent forced I-Frame");
     else if (m_consecutiveIntraFrames < 10) {
       if (++m_consecutiveIntraFrames >= 10)
-        PTRACE(3, "OpalPlugin\tSending many consecutive I-Frames, assuming codec cannot do P-Frames");
+        PTRACE(3, "OpalPlugin\tEncoder has sent too many consecutive I-Frames - assume codec cannot do P-Frames");
       else
-        PTRACE(4, "OpalPlugin\tSending I-Frame");
+        PTRACE(4, "OpalPlugin\tEncoder resending forced I-Frame");
     }
 #endif
 
@@ -1035,7 +1036,7 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
     if (!Transcode((const BYTE *)src, &fromLen, m_bufferRTP->GetPointer(), &toLen, &flags))
       return false;
 
-    if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0 && commandNotifier != PNotifier()) {
+    if ((commandNotifier != PNotifier()) && (flags & PluginCodec_ReturnCoderRequestIFrame) != 0) {
       PTimeInterval tick = PTimer::Tick();
       // Don't send lots of consecutive VideoFastUpdate commands
       if (tick - m_lastVideoFastUpdate < 2000)
@@ -1048,18 +1049,34 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
       }
     }
 
-    if (toLen > (unsigned)m_bufferRTP->GetHeaderSize() && (flags & PluginCodec_ReturnCoderLastFrame) != 0) {
-      m_bufferRTP->SetPayloadSize(toLen);
-      dstList.Append(m_bufferRTP);
-      m_bufferRTP = NULL;
+    if (toLen > RTP_DataFrame::MinHeaderSize && (flags & PluginCodec_ReturnCoderLastFrame) != 0) {
+      // Do sanity check on returned data.
+      PINDEX headerSize = m_bufferRTP->GetHeaderSize();
+      if (PAssert(toLen > (unsigned)(headerSize+sizeof(OpalVideoTranscoder::FrameHeader)),
+                  "Invalid return size from video plug in")) {
+        toLen -= headerSize;
+        OpalVideoTranscoder::FrameHeader * videoHeader = (OpalVideoTranscoder::FrameHeader *)m_bufferRTP->GetPayloadPtr();
+        if (PAssert(videoHeader->x == 0 && videoHeader->y == 0 &&
+                    videoHeader->width < 10000 && videoHeader->height < 10000 &&
+                    toLen >= (videoHeader->width*videoHeader->height*3/2+sizeof(OpalVideoTranscoder::FrameHeader)),
+                    "Invalid frame returned from video plug in")) {
+          m_bufferRTP->SetPayloadSize(toLen);
+          m_bufferRTP->SetPayloadType(GetPayloadType(false));
+          dstList.Append(m_bufferRTP);
+          m_bufferRTP = NULL;
 
-      lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
+          lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
+          if (lastFrameWasIFrame) {
+            PTRACE(5, "OpalPlugin\tVideo decoder returned I-frame");
+          }
 
 #if OPAL_STATISTICS
-      m_totalFrames++;
-      if (lastFrameWasIFrame)
-        m_keyFrames++;
+          m_totalFrames++;
+          if (lastFrameWasIFrame)
+            m_keyFrames++;
 #endif
+        }
+      }
     }
   }
 
