@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22086 $
+ * $Revision: 22563 $
  * $Author: csoutheren $
- * $Date: 2009-02-18 03:37:22 +0000 (Wed, 18 Feb 2009) $
+ * $Date: 2009-05-07 17:34:54 +0000 (Thu, 07 May 2009) $
  */
 
 #include <ptlib.h>
@@ -397,13 +397,13 @@ void OpalMediaOptionEnum::SetValue(PINDEX value)
 ///////////////////////////////////////
 
 OpalMediaOptionString::OpalMediaOptionString(const char * name, bool readOnly)
-  : OpalMediaOption(name, readOnly, MinMerge)
+  : OpalMediaOption(name, readOnly, NoMerge)
 {
 }
 
 
 OpalMediaOptionString::OpalMediaOptionString(const char * name, bool readOnly, const PString & value)
-  : OpalMediaOption(name, readOnly, MinMerge),
+  : OpalMediaOption(name, readOnly, NoMerge),
     m_value(value)
 {
 }
@@ -419,7 +419,7 @@ PObject * OpalMediaOptionString::Clone() const
 
 void OpalMediaOptionString::PrintOn(ostream & strm) const
 {
-  strm << m_value.ToLiteral();
+  strm << m_value;
 }
 
 
@@ -631,14 +631,14 @@ const PString & OpalMediaFormat::FrameTimeOption()     { static PString s = PLUG
 const PString & OpalMediaFormat::ClockRateOption()     { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;      return s; }
 const PString & OpalMediaFormat::MaxBitRateOption()    { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;    return s; }
 const PString & OpalMediaFormat::TargetBitRateOption() { static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
-const PString & OpalMediaFormat::BandwidthTIASOption() { static PString s = PLUGINCODEC_OPTION_TIAS;            return s; }
-const PString & OpalMediaFormat::MaxPacketRateOption() { static PString s = PLUGINCODEC_OPTION_MAXPRATE;        return s; }
 
 #if OPAL_H323
 const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s = PLUGINCODEC_MEDIA_PACKETIZATION;  return s; }
 const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
 #endif
 
+const PString & OpalMediaFormat::ProtocolOption()        { static PString s = "Protocol";           return s; }
+const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static PString s = "Max Tx Packet Size"; return s; }
 
 OpalMediaFormat::OpalMediaFormat(OpalMediaFormatInternal * info)
   : m_info(NULL)
@@ -902,7 +902,7 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
   if (cr > 0)
     AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::NoMerge, cr));
 
-  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::BandwidthTIASOption(), false, OpalMediaOption::MinMerge, bw, 1, bw));
+  AddOption(new OpalMediaOptionString(OpalMediaFormat::ProtocolOption(), true));
 
   // assume non-dynamic payload types are correct and do not need deconflicting
   if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType) {
@@ -1419,9 +1419,7 @@ const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static PStri
 const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static PString s = PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF;return s; }
 const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static PString s = PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD;       return s; }
 const PString & OpalVideoFormat::RateControlEnableOption()        { static PString s = "Rate Control Enable";                        return s; }
-const PString & OpalVideoFormat::RateControlWindowSizeOption()    { static PString s = "Rate Control Window Size";                   return s; }
-const PString & OpalVideoFormat::RateControlMaxFramesSkipOption() { static PString s = "Rate Control Max Frames Skip";               return s; }
-const PString & OpalVideoFormat::RateControllerBitRateScalerOption() { static PString s = "Rate Controller Bit Rate Scaler";            return s; }
+const PString & OpalVideoFormat::RateControllerOption()           { static PString s = "Rate Controller";                            return s; }
 
 OpalVideoFormat::OpalVideoFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1471,10 +1469,9 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,   1000));
   AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     5000,                        100,100000));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     5,                           1,   10   ));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControllerBitRateScalerOption(), false, OpalMediaOption::NoMerge,  100,                         50, 200));
-								
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          false, OpalMediaOption::NoMerge,     2048,                        100,  2048));
+  AddOption(new OpalMediaOptionString  (OpalVideoFormat::RateControllerOption(),           false                                                                       ));
+
   // For video the max bit rate and frame rate is adjustable by user
   FindOption(OpalVideoFormat::MaxBitRateOption())->SetReadOnly(false);
   FindOption(OpalVideoFormat::FrameTimeOption())->SetReadOnly(false);
@@ -1520,6 +1517,7 @@ OpalMediaFormatList::OpalMediaFormatList(const OpalMediaFormat & format)
 
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & format)
 {
+  MakeUnique();
   if (format.IsValid() && !HasFormat(format))
     OpalMediaFormatBaseList::Append(format.Clone());
   return *this;
@@ -1528,6 +1526,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & fo
 
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormatList & formats)
 {
+  MakeUnique();
   for (OpalMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format)
     *this += *format;
   return *this;
@@ -1536,6 +1535,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormatList
 
 OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormat & format)
 {
+  MakeUnique();
   OpalMediaFormatList::const_iterator fmt = FindFormat(format);
   if (fmt != end())
     erase(fmt);
@@ -1546,6 +1546,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormat & fo
 
 OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormatList & formats)
 {
+  MakeUnique();
   for (OpalMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format)
     *this -= *format;
   return *this;
@@ -1564,6 +1565,7 @@ void OpalMediaFormatList::Remove(const PStringArray & maskList)
   }
 }
 
+
 OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol) const
 {
   OpalMediaFormatList::const_iterator format;
@@ -1671,16 +1673,30 @@ void OpalMediaFormatList::Reorder(const PStringArray & order)
   DisallowDeleteObjects();
   PINDEX nextPos = 0;
   for (PINDEX i = 0; i < order.GetSize(); i++) {
-    PStringArray wildcards = order[i].Tokenise('*', true);
+    if (order[i][0] == '@') {
+      OpalMediaType mediaType = order[i].Mid(1);
+      PINDEX findPos = 0;
+      while (findPos < GetSize()) {
+        if ((*this)[findPos].GetMediaType() == mediaType) {
+          if (findPos > nextPos)
+            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
+          nextPos++;
+        }
+        findPos++;
+      }
+    }
+    else {
+      PStringArray wildcards = order[i].Tokenise('*', true);
 
-    PINDEX findPos = 0;
-    while (findPos < GetSize()) {
-      if (WildcardMatch((*this)[findPos].m_info->formatName, wildcards)) {
-        if (findPos > nextPos)
-          OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
-        nextPos++;
+      PINDEX findPos = 0;
+      while (findPos < GetSize()) {
+        if (WildcardMatch((*this)[findPos].m_info->formatName, wildcards)) {
+          if (findPos > nextPos)
+            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
+          nextPos++;
+        }
+        findPos++;
       }
-      findPos++;
     }
   }
   AllowDeleteObjects();
