@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 22122 $
- * $Author: csoutheren $
- * $Date: 2009-02-26 11:44:57 +0000 (Thu, 26 Feb 2009) $
+ * $Revision: 22640 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
  */
 
 #include <ptlib.h>
@@ -249,7 +249,7 @@ PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
   unsigned oldTimestamp = timestamp;
 
-  PAssert(defaultDataSize >= (packet.GetSize() - RTP_DataFrame::MinHeaderSize), "buffer for media packet is too small");
+  PAssert(defaultDataSize <= (packet.GetSize() - packet.GetHeaderSize()), "default data size too large for packet");
 
   PINDEX lastReadCount;
   if (!ReadData(packet.GetPayloadPtr(), defaultDataSize, lastReadCount))
@@ -433,11 +433,8 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
   if (!safeLock.IsLocked())
     return false;
 
-  if (IsOpen()) {
-    mediaPatch = patch;
-    return true;
-  }
-  return false;
+  mediaPatch = patch;
+  return true;
 }
 
 
@@ -465,14 +462,14 @@ PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMedia
 
 
 #if OPAL_STATISTICS
-void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   PSafeLockReadOnly safeLock(*this);
   if (!safeLock.IsLocked())
     return;
 
-  if (mediaPatch != NULL)
-    mediaPatch->GetStatistics(statistics);
+  if (mediaPatch != NULL && !fromPatch)
+    mediaPatch->GetStatistics(statistics, IsSink());
 }
 #endif
 
@@ -595,13 +592,12 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
      whatever the RTP sender throws at us. For sink, we set it to the
      maximum size based on MTU (or other criteria). */
   defaultDataSize = isSource ? 2048 : conn.GetMaxRtpPayloadSize();
-
-  rtp.SetEncoding(mediaFormat.GetMediaType().GetDefinition()->GetRTPEncoding());
 }
 
 
 OpalRTPMediaStream::~OpalRTPMediaStream()
 {
+  Close();
 }
 
 
@@ -610,6 +606,7 @@ PBoolean OpalRTPMediaStream::Open()
   if (isOpen)
     return true;
 
+  rtpSession.SetEncoding(mediaFormat.GetMediaType().GetDefinition()->GetRTPEncoding());
   rtpSession.Reopen(IsSource());
 
   return OpalMediaStream::Open();
@@ -660,11 +657,7 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
   if (packet.GetPayloadSize() == 0)
     return true;
 
-  PBoolean ret;
-  ret = rtpSession.WriteData(packet);
-
-  return ret;
-
+  return rtpSession.WriteData(packet);
 }
 
 
@@ -694,10 +687,10 @@ void OpalRTPMediaStream::EnableJitterBuffer() const
 
 
 #if OPAL_STATISTICS
-void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   rtpSession.GetStatistics(statistics, IsSource());
-  OpalMediaStream::GetStatistics(statistics);
+  OpalMediaStream::GetStatistics(statistics, fromPatch);
 }
 #endif
 
@@ -832,13 +825,13 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
 
 PBoolean OpalRawMediaStream::Close()
 {
-  if (!OpalMediaStream::Close())
+  if (!isOpen)
     return false;
 
   if (m_channel != NULL)
     m_channel->Close();
 
-  return true;
+  return OpalMediaStream::Close();
 }
 
 
@@ -1009,6 +1002,7 @@ OpalVideoMediaStream::OpalVideoMediaStream(OpalConnection & conn,
 
 OpalVideoMediaStream::~OpalVideoMediaStream()
 {
+  Close();
   if (autoDelete) {
     delete inputDevice;
     delete outputDevice;
