@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22078 $
- * $Author: rjongbloed $
- * $Date: 2009-02-16 23:46:57 +0000 (Mon, 16 Feb 2009) $
+ * $Revision: 22553 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
  */
 
 #include <ptlib.h>
@@ -290,21 +290,28 @@ void OpalMediaPatch::UnLockSinkTranscoder() const
 
 
 #if OPAL_STATISTICS
-void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const
 {
   inUse.StartRead();
 
+  if (fromSink)
+    source.GetStatistics(statistics, true);
+
   if (!sinks.IsEmpty())
-    sinks.front().GetStatistics(statistics);
+    sinks.front().GetStatistics(statistics, !fromSink);
 
   inUse.EndRead();
 }
 
 
-void OpalMediaPatch::Sink::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaPatch::Sink::GetStatistics(OpalMediaStatistics & statistics, bool fromSource) const
 {
+  if (fromSource)
+    stream->GetStatistics(statistics, true);
+
   if (primaryCodec != NULL)
     primaryCodec->GetStatistics(statistics);
+
   if (secondaryCodec != NULL)
     secondaryCodec->GetStatistics(statistics);
 }
@@ -318,7 +325,7 @@ OpalMediaPatch::Sink::Sink(OpalMediaPatch & p, const OpalMediaStreamPtr & s)
   , secondaryCodec(NULL)
   , writeSuccessful(true)
 #if OPAL_VIDEO
-  , rcEnabled(false)
+  , rateController(NULL)
 #endif
 {
 #if OPAL_VIDEO
@@ -333,6 +340,9 @@ OpalMediaPatch::Sink::~Sink()
 {
   delete primaryCodec;
   delete secondaryCodec;
+#if OPAL_VIDEO
+  delete rateController;
+#endif
 }
 
 
@@ -449,11 +459,12 @@ void OpalMediaPatch::Main()
 	
   bool asynchronous = OnPatchStart();
 
-  RTP_DataFrame sourceFrame(source.GetDataSize());
-  sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
-
   while (source.IsOpen()) {
+
+    RTP_DataFrame sourceFrame(source.GetDataSize());
+    sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
     sourceFrame.SetPayloadSize(0); 
+
     if (!source.ReadPacket(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because source read failed");
       break;
@@ -567,8 +578,10 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
     return true;
   }
 
-  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) 
+  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
+    frame.SetPayloadType(codec.GetPayloadType(false));
     return true;
+  }
 
   return false;
 }
@@ -577,27 +590,30 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
 #if OPAL_VIDEO
 void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & mediaFormat)
 {
-  rcEnabled = (mediaFormat.GetMediaType() == OpalMediaType::Video()) &&
-              mediaFormat != OpalYUV420P &&
-              mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption());
-
-  if (rcEnabled) {
-    unsigned targetBitRate = mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption());
-    if (mediaFormat.HasOption(OpalVideoFormat::RateControllerBitRateScalerOption()))
-      targetBitRate = targetBitRate * mediaFormat.GetOptionInteger(OpalVideoFormat::RateControllerBitRateScalerOption()) / 100;
-    rateController.Open(
-                        targetBitRate,
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::FrameTimeOption(), -1),
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlWindowSizeOption()),
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlMaxFramesSkipOption())
-                        );
+  if ((mediaFormat.GetMediaType() == OpalMediaType::Video()) && mediaFormat != OpalYUV420P) {
+    rateController = NULL;
+    PString rc = mediaFormat.GetOptionString(OpalVideoFormat::RateControllerOption());
+    if (rc.IsEmpty() && mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption()))
+      rc = "Standard";
+    if (!rc.IsEmpty()) {
+      rateController = PFactory<OpalVideoRateController>::CreateInstance(rc);
+      if (rateController != NULL) {   
+        PTRACE(3, "Patch\tCreated " << rc << " rate controller");
+      }
+      else {
+        PTRACE(3, "Patch\tCould not create " << rc << " rate controller");
+      }
+    }
   }
+
+  if (rateController != NULL) 
+    rateController->Open(mediaFormat);
 }
 
 
-bool OpalMediaPatch::Sink::RateControlExceeded()
+bool OpalMediaPatch::Sink::RateControlExceeded(bool & forceIFrame)
 {
-  if (!rcEnabled || !rateController.SkipFrame()) 
+  if ((rateController == NULL) || !rateController->SkipFrame(forceIFrame)) 
     return false;
 
   PTRACE(4, "Patch\tRate controller skipping frame.");
@@ -613,11 +629,29 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     return false;
   
 #if OPAL_VIDEO
-  if (RateControlExceeded())
-    return true;
-
-  PInt64 totalPayloadSize = 0;
-  int frameCount = 0;
+  if (rateController != NULL) {
+    bool forceIFrame = false;
+    bool s = RateControlExceeded(forceIFrame);
+    if (forceIFrame)
+      stream->ExecuteCommand(OpalVideoUpdatePicture());
+    if (s) {
+      if (secondaryCodec == NULL) {
+        bool wasIFrame = false;
+        if (rateController->Pop(intermediateFrames, wasIFrame, false)) {
+        PTRACE(3, "RC returned " << intermediateFrames.GetSize() << " packets");
+          for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
+            patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
+            if (!stream->WritePacket(*interFrame))
+              return (writeSuccessful = false);
+            sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+            continue;
+          }
+          intermediateFrames.RemoveAll();
+        }
+      }
+      return true;
+    }
+  }
 #endif
 
   if (primaryCodec == NULL)
@@ -631,16 +665,31 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     return false;
   }
 
+#if OPAL_VIDEO
+  if (secondaryCodec == NULL && rateController != NULL) {
+    PTRACE(4, "Patch\tPushing " << intermediateFrames.GetSize() << " packet into RC");
+    rateController->Push(intermediateFrames, ((OpalVideoTranscoder *)primaryCodec)->WasLastFrameIFrame());
+    bool wasIFrame = false;
+    if (rateController->Pop(intermediateFrames, wasIFrame, false)) {
+      PTRACE(4, "Patch\tPulled " << intermediateFrames.GetSize() << " frames from RC");
+      for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
+        patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
+        if (!stream->WritePacket(*interFrame))
+          return (writeSuccessful = false);
+        sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+        continue;
+      }
+      intermediateFrames.RemoveAll();
+    }
+  }
+  else 
+#endif
   for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
     patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
 
     if (secondaryCodec == NULL) {
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
-#if OPAL_VIDEO
-      totalPayloadSize += interFrame->GetPayloadSize() + interFrame->GetHeaderSize();
-      frameCount++;
-#endif
       sourceFrame.SetTimestamp(interFrame->GetTimestamp());
       continue;
     }
@@ -660,17 +709,13 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       patch.FilterFrame(*finalFrame, secondaryCodec->GetOutputFormat());
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
-#if OPAL_VIDEO
-      totalPayloadSize += finalFrame->GetPayloadSize() + finalFrame->GetHeaderSize();
-      frameCount++;
-#endif
       sourceFrame.SetTimestamp(finalFrame->GetTimestamp());
     }
   }
 
 #if OPAL_VIDEO
-  if (rcEnabled)
-    rateController.AddFrame(totalPayloadSize, frameCount);
+  //if (rcEnabled)
+  //  rateController.AddFrame(totalPayloadSize, frameCount);
 #endif
 
   return true;
@@ -704,8 +749,8 @@ void OpalPassiveMediaPatch::Start()
 
 PBoolean OpalPassiveMediaPatch::PushFrame(RTP_DataFrame & frame)
 {
-  DispatchFrame(frame);
-  return PTrue;
+  PReadWaitAndSignal mutex(inUse);
+  return DispatchFrame(frame);
 }
 
 
