@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21406 $
- * $Author: csoutheren $
- * $Date: 2008-10-22 11:59:46 +0000 (Wed, 22 Oct 2008) $
+ * $Revision: 22352 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-31 07:14:52 +0000 (Tue, 31 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -244,6 +244,7 @@ PBoolean OpalJitterBuffer::Init(Entry * & /*currentReadFrame*/, PBoolean & marke
 
 void OpalJitterBuffer::DeInit(Entry * & /*currentReadFrame*/, PBoolean & /*markerWarning*/)
 {
+  bufferMutex.Signal();
 }
 
 PBoolean OpalJitterBuffer::PreRead(OpalJitterBuffer::Entry * & currentReadFrame, PBoolean & /*markerWarning*/)
@@ -290,6 +291,7 @@ PBoolean OpalJitterBuffer::OnRead(OpalJitterBuffer::Entry * & currentReadFrame,
   do {
     // Keep reading from the RTP transport frames
     if (!OnReadPacket(*currentReadFrame, loop)) {
+      bufferMutex.Wait();
       if (currentReadFrame != NULL)
         delete currentReadFrame;  // Destructor won't delete this one, so do it here.
       shuttingDown = true; // Flag to stop the reading side thread
