@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22097 $
+ * $Revision: 22589 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-05-12 10:18:55 +0000 (Tue, 12 May 2009) $
  */
 
 #include <ptlib.h>
@@ -109,7 +109,7 @@ ReasonToSIPCode[] = {
   { SIP_PDU::Failure_Forbidden                  , OpalConnection::EndedByNoUser            ,  55 }, // incoming calls barred within CUG     
   { SIP_PDU::Failure_Forbidden                  , OpalConnection::EndedByNoUser            ,  57 }, // bearer capability not authorized     
   { SIP_PDU::Failure_ServiceUnavailable         , OpalConnection::EndedByNoUser            ,  58 }, // bearer capability not presently available
-  { SIP_PDU::Failure_NotAcceptableHere          , OpalConnection::EndedByNoUser            ,  65 }, // bearer capability not implemented
+  { SIP_PDU::Failure_NotAcceptableHere          , OpalConnection::EndedByCapabilityExchange,  65 }, // bearer capability not implemented
   { SIP_PDU::Failure_NotAcceptableHere          , OpalConnection::EndedByNoUser            ,  70 }, // only restricted digital avail    
   { SIP_PDU::Failure_NotImplemented             , OpalConnection::EndedByNoUser            ,  79 }, // service or option not implemented
   { SIP_PDU::Failure_Forbidden                  , OpalConnection::EndedByNoUser            ,  87 }, // user not member of CUG           
@@ -118,7 +118,6 @@ ReasonToSIPCode[] = {
   { SIP_PDU::Failure_InternalServerError        , OpalConnection::EndedByNoUser            , 111 }, // protocol error                   
   { SIP_PDU::Failure_InternalServerError        , OpalConnection::EndedByNoUser            , 127 }, // interworking unspecified         
   { SIP_PDU::Failure_RequestTerminated          , OpalConnection::EndedByCallerAbort             },
-  { SIP_PDU::Failure_UnsupportedMediaType       , OpalConnection::EndedByCapabilityExchange      },
   { SIP_PDU::Redirection_MovedTemporarily       , OpalConnection::EndedByCallForwarded           },
   { SIP_PDU::GlobalFailure_Decline              , OpalConnection::EndedByAnswerDenied            },
   { SIP_PDU::GlobalFailure_Decline              , OpalConnection::EndedByRefusal                 }, // TODO - SGW - add for call reject from H323 side.
@@ -146,6 +145,7 @@ SIPCodeToReason[] = {
   { SIP_PDU::Failure_RequestEntityTooLarge      , OpalConnection::EndedByQ931Cause         , 127 }, // Interworking (+)
   { SIP_PDU::Failure_RequestURITooLong          , OpalConnection::EndedByQ931Cause         , 127 }, // Interworking (+)
   { SIP_PDU::Failure_UnsupportedMediaType       , OpalConnection::EndedByCapabilityExchange,  79 }, // Service/option not implemented (+)
+  { SIP_PDU::Failure_NotAcceptableHere          , OpalConnection::EndedByCapabilityExchange,  79 }, // Service/option not implemented (+)
   { SIP_PDU::Failure_UnsupportedURIScheme       , OpalConnection::EndedByQ931Cause         , 127 }, // Interworking (+)
   { SIP_PDU::Failure_BadExtension               , OpalConnection::EndedByQ931Cause         , 127 }, // Interworking (+)
   { SIP_PDU::Failure_ExtensionRequired          , OpalConnection::EndedByQ931Cause         , 127 }, // Interworking (+)
@@ -303,6 +303,11 @@ void SIPConnection::OnReleased()
 
       // EndedByCallForwarded is a special case because it needs extra paramater
       SendInviteResponse(sipCode, NULL, callEndReason == EndedByCallForwarded ? (const char *)forwardParty : NULL);
+
+      // Wait for ACK from remote before destroying object
+      while (!ackReceived)
+        PThread::Sleep(100);
+
       notifyDialogEvent = SIPDialogNotification::Rejected;
       break;
 
@@ -416,8 +421,6 @@ PBoolean SIPConnection::SetAlerting(const PString & /*calleeName*/, PBoolean wit
   else {
     SDPSessionDescription sdpOut(GetDefaultSDPConnectAddress());
     if (!OnSendSDP(true, m_rtpSessions, sdpOut)) {
-      SendInviteResponse(SIP_PDU::Failure_NotAcceptableHere);
-      releaseMethod = ReleaseWithNothing;
       Release(EndedByCapabilityExchange);
       return PFalse;
     }
@@ -456,8 +459,6 @@ PBoolean SIPConnection::SetConnected()
 
   SDPSessionDescription sdpOut(GetDefaultSDPConnectAddress());
   if (!OnSendSDP(true, m_rtpSessions, sdpOut)) {
-    SendInviteResponse(SIP_PDU::Failure_NotAcceptableHere);
-    releaseMethod = ReleaseWithNothing;
     Release(EndedByCapabilityExchange);
     return PFalse;
   }
@@ -571,7 +572,8 @@ PBoolean SIPConnection::OnSendSDP(bool isAnswerSDP, OpalRTPSessionManager & rtpS
     }
   }
 
-  needReINVITE = true;
+  if (isAnswerSDP)
+    needReINVITE = true;
 
   return sdpOK;
 }
@@ -1049,6 +1051,8 @@ OpalMediaFormatList SIPConnection::GetMediaFormats() const
 
 OpalMediaStreamPtr SIPConnection::OpenMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, bool isSource)
 {
+  PTRACE(1, "SIPConnection::OpenMediaStream called with " << mediaFormat << ", " << sessionID << ", " << isSource);
+
   // Disable send of re-INVITE if the ancestor OpenMediaStream has to close
   // the media stream, so don't get two re-INVITEs in quick succession.
   bool oldReINVITE = needReINVITE;
@@ -1079,10 +1083,14 @@ OpalMediaStreamPtr SIPConnection::OpenMediaStream(const OpalMediaFormat & mediaF
     return newStream;
   }
 
-  // Open other direction, if needed
-  if (makesymmetrical)
-    GetCall().OpenSourceMediaStreams(*(isSource ? GetCall().GetOtherPartyConnection(*this) : this),
-                                       mediaFormat.GetMediaType(), sessionID, mediaFormat);
+  // Open other direction, if needed (must be after above open)
+  if (makesymmetrical &&
+      !GetCall().OpenSourceMediaStreams(*(isSource ? GetCall().GetOtherPartyConnection(*this) : this),
+                                                  mediaFormat.GetMediaType(), sessionID, mediaFormat)) {
+    newStream->Close();
+    needReINVITE = oldReINVITE;
+    return NULL;
+  }
 
   needReINVITE = oldReINVITE;
 
@@ -1119,7 +1127,9 @@ PBoolean SIPConnection::WriteINVITE(OpalTransport & transport, void * param)
 bool SIPConnection::WriteINVITE(OpalTransport & transport)
 {
   const SIPURL & requestURI = m_dialog.GetRequestURI();
-  SIPURL myAddress = endpoint.GetRegisteredPartyName(requestURI, transport);
+  SIPURL myAddress = m_connStringOptions(OPAL_OPT_CALLING_PARTY_URL);
+  if (myAddress.IsEmpty())
+    myAddress = endpoint.GetRegisteredPartyName(requestURI, transport);
 
   PString transportProtocol = requestURI.GetParamVars()("transport");
   if (!transportProtocol.IsEmpty())
@@ -1128,15 +1138,15 @@ bool SIPConnection::WriteINVITE(OpalTransport & transport)
   // only allow override of calling party number if the local party
   // name hasn't been first specified by a register handler. i.e a
   // register handler's target number is always used
-  PString number(m_connStringOptions("Calling-Party-Number"));
-  if (!number.IsEmpty() && myAddress.GetUserName() == endpoint.GetDefaultLocalPartyName())
+  PString number(m_connStringOptions(OPAL_OPT_CALLING_PARTY_NUMBER));
+  if (!number.IsEmpty())
     myAddress.SetUserName(number);
 
-  PString name(m_connStringOptions("Calling-Party-Name"));
+  PString name(m_connStringOptions(OPAL_OPT_CALLING_PARTY_NAME));
   if (!name.IsEmpty())
     myAddress.SetDisplayName(name);
 
-  PString domain(m_connStringOptions("Calling-Party-Domain"));
+  PString domain(m_connStringOptions(OPAL_OPT_CALLING_PARTY_DOMAIN));
   if (!domain.IsEmpty())
     myAddress.SetHostName(domain);
 
@@ -1320,7 +1330,7 @@ PString SIPConnection::GetIdentifier() const
 
 PString SIPConnection::GetRemotePartyURL() const
 {
-  SIPURL url = GetRemotePartyAddress();
+  SIPURL url = m_dialog.GetRemoteURI();
   url.Sanitise(SIPURL::ExternalURI);
   return url.AsString();
 }
@@ -1392,8 +1402,6 @@ void SIPConnection::OnReceivedPDU(SIP_PDU & pdu)
   }
   m_lastRxCSeq[method] = sequenceNumber;
 
-  PTRACE(4, "SIP\tHandling PDU " << pdu);
-
   switch (method) {
     case SIP_PDU::Method_INVITE :
       OnReceivedINVITE(pdu);
@@ -1484,7 +1492,7 @@ void SIPConnection::UpdateRemoteAddresses()
   SIPURL url = m_dialog.GetRemoteURI();
   url.Sanitise(SIPURL::ExternalURI);
 
-  remotePartyAddress = url.AsString();
+  remotePartyAddress = url.GetHostAddress();
 
   remotePartyNumber = url.GetUserName();
   if (remotePartyNumber.FindSpan("0123456789*#") != P_MAX_INDEX)
@@ -1722,7 +1730,7 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
   m_calledPartyName = request.GetURI().GetUserName();
   if (!m_calledPartyName.IsEmpty() && m_calledPartyName.FindSpan("0123456789*#") == P_MAX_INDEX) {
     m_calledPartyNumber = m_calledPartyName;
-    m_calledPartyName = request.GetURI().GetDisplayName();
+    m_calledPartyName = request.GetURI().GetDisplayName(false);
   }
 
   // get the address that remote end *thinks* it is using from the Contact field
@@ -1739,17 +1747,19 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
 
   releaseMethod = ReleaseWithResponse;
 
-  PTRACE(3, "SIP\tOnIncomingConnection succeeded for INVITE from " << request.GetURI() << " for " << *this);
   SetPhase(SetUpPhase);
 
   // Replaces header has already been validated in SIPEndPoint::OnReceivedINVITE
-  PString replaces = mime("Replaces");
-  PSafePtr<SIPConnection> replacedConnection = endpoint.GetSIPConnectionWithLock(replaces.Left(replaces.Find(';')).Trim(), PSafeReadOnly);
+  PSafePtr<SIPConnection> replacedConnection = endpoint.GetSIPConnectionWithLock(mime("Replaces"), PSafeReadOnly);
   if (replacedConnection != NULL) {
     PTRACE(3, "SIP\tConnection " << *replacedConnection << " replaced by " << *this);
 
     replacedConnection->Release(OpalConnection::EndedByCallForwarded);
-    replacedConnection->CloseMediaStreams();
+
+    // Before proceeding with connection, wait for streams to all be closed
+    while (replacedConnection->GetMediaStream(PString::Empty(), true) != NULL &&
+           replacedConnection->GetMediaStream(PString::Empty(), false) != NULL)
+      PThread::Sleep(20);
 
     // in some circumstances, the peer OpalConnection needs to see the newly arrived media formats
     // before it knows what what formats can support. 
@@ -1766,6 +1776,8 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
     return;
   }
 
+  PTRACE(3, "SIP\tOnIncomingConnection succeeded for INVITE from " << request.GetURI() << " for " << *this);
+
   // in some circumstances, the peer OpalConnection needs to see the newly arrived media formats
   // before it knows what what formats can support. 
   GetOtherPartyConnection()->PreviewPeerMediaFormats(GetMediaFormats());
@@ -2048,7 +2060,7 @@ void SIPConnection::OnReceivedRedirection(SIP_PDU & response)
 {
   m_dialog.Update(response);
   UpdateRemoteAddresses();
-  endpoint.ForwardConnection (*this, remotePartyAddress);
+  endpoint.ForwardConnection (*this, GetRemotePartyURL());
 }
 
 
@@ -2110,7 +2122,6 @@ PBoolean SIPConnection::OnReceivedAuthenticationRequired(SIPTransaction & transa
   delete authentication;
   authentication = newAuth;
 
-  needReINVITE = false; // Is not actually a re-INVITE though it looks a little bit like one.
   transport->SetInterface(transaction.GetInterface());
   SIPTransaction * invite = new SIPInvite(*this, *transport, ((SIPInvite &)transaction).GetSessionManager());
 
@@ -2231,17 +2242,18 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
   }
   PTRACE(4, "SIP\tProcessing received SDP media description for " << mediaType);
 
-  // see if the remote supports this media
+  /* Get the media the remote has answered to our offer. Remove the media
+     formats we do not support, in case the remote is insane and replied
+     with something we did not actually offer. */
   OpalMediaFormatList mediaFormatList = mediaDescription->GetMediaFormats();
+  mediaFormatList.Remove(endpoint.GetManager().GetMediaFormatMask());
   if (mediaFormatList.GetSize() == 0) {
-    PTRACE(1, "SIP\tCould not find media formats in SDP media description for session " << rtpSessionId);
+    PTRACE(2, "SIP\tCould not find supported media formats in SDP media description for session " << rtpSessionId);
     return false;
   }
 
   // When receiving an answer SDP, keep the remote SDP media formats order
-  // but remove the media formats we do not support.
   remoteFormatList += mediaFormatList;
-  remoteFormatList.Remove(endpoint.GetManager().GetMediaFormatMask());
 
   // Find the payload type and capabilities used for telephone-event, if present
   remoteFormatList += GetNxECapabilities(rfc2833Handler, mediaDescription, OpalRFC2833);
@@ -2250,6 +2262,7 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
 #endif
 
   // create the RTPSession
+  bool remoteChanged = false;
   OpalTransportAddress localAddress;
   OpalTransportAddress address = mediaDescription->GetTransportAddress();
   if (!address.IsEmpty()) {
@@ -2271,10 +2284,18 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
       // set the remote address 
       PIPSocket::Address ip;
       WORD port = 0;
-      if (!address.GetIpAndPort(ip, port) || port == 0 || (rtpSession && !rtpSession->SetRemoteSocketInfo(ip, port, true))) {
+      if (!address.GetIpAndPort(ip, port) || port == 0) {
+        PTRACE(1, "SIP\tCannot get remote ip/ports for RTP session");
+        return false;
+      }
+
+      // see if remote socket information has changed
+      remoteChanged = (rtpSession->GetRemoteAddress() != ip) || (rtpSession->GetRemoteDataPort() != port);
+      if (remoteChanged && !rtpSession->SetRemoteSocketInfo(ip, port, true)) {
         PTRACE(1, "SIP\tCannot set remote ports on RTP session");
         return false;
       }
+      PTRACE(4, "SIP\tRemote changed IP address");
     }
   }
 
@@ -2284,7 +2305,7 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
   // changed the direction of the stream
   OpalMediaStreamPtr sendStream = GetMediaStream(rtpSessionId, false);
   if (sendStream != NULL && sendStream->IsOpen()) {
-    if (mediaFormatList.HasFormat(sendStream->GetMediaFormat()))
+    if (!remoteChanged && mediaFormatList.HasFormat(sendStream->GetMediaFormat()))
       sendStream->SetPaused((otherSidesDir&SDPMediaDescription::RecvOnly) == 0);
     else {
       sendStream->GetPatch()->GetSource().Close(); // Was removed from list so close channel
@@ -2294,7 +2315,7 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
 
   OpalMediaStreamPtr recvStream = GetMediaStream(rtpSessionId, true);
   if (recvStream != NULL && recvStream->IsOpen()) {
-    if (mediaFormatList.HasFormat(recvStream->GetMediaFormat()))
+    if (!remoteChanged && mediaFormatList.HasFormat(recvStream->GetMediaFormat()))
       recvStream->SetPaused((otherSidesDir&SDPMediaDescription::SendOnly) == 0);
     else {
       recvStream->Close(); // Was removed from list so close channel
@@ -2303,12 +2324,14 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
   }
 
   // Then open the streams if the direction allows
-  if ((otherSidesDir&SDPMediaDescription::SendOnly) != 0)
+  if (recvStream == NULL && (otherSidesDir&SDPMediaDescription::SendOnly) != 0)
     ownerCall.OpenSourceMediaStreams(*this, mediaType, rtpSessionId);
 
-  PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
-  if ((otherSidesDir&SDPMediaDescription::RecvOnly) != 0 && otherParty != NULL)
-    ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, rtpSessionId);
+  if (sendStream == NULL && (otherSidesDir&SDPMediaDescription::RecvOnly) != 0) {
+    PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
+    if (otherParty != NULL)
+      ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, rtpSessionId);
+  }
 
   PTRACE_IF(3, otherSidesDir == SDPMediaDescription::Inactive, "SIP\tNo streams opened as " << mediaType << " inactive");
   return true;
@@ -2351,14 +2374,11 @@ PBoolean SIPConnection::ForwardCall (const PString & fwdParty)
 
 PBoolean SIPConnection::SendInviteOK(const SDPSessionDescription & sdp)
 {
-  const SIPURL & localPartyURL = m_dialog.GetLocalURI();
-  PString userName = endpoint.GetRegisteredPartyName(localPartyURL, *transport).GetUserName();
-
   // this can be used to prompoe any incoming calls to TCP. Not quite there yet, but it *almost* works
   SIPURL contact;
   bool promoteToTCP = false;    // disable code for now
   if (!promoteToTCP || (transport != NULL && (PString(transport->GetProtoPrefix()) == "$tcp"))) 
-    contact = endpoint.GetContactURL(*transport, userName, localPartyURL.GetHostName());
+    contact = endpoint.GetContactURL(*transport, m_dialog.GetLocalURI());
   else {
     // see if endpoint contains a TCP listener we can use
     OpalTransportAddress newAddr;
@@ -2410,8 +2430,11 @@ void SIPConnection::OnAckTimeout(PTimer &, INT)
   if (safeLock.IsLocked() && !ackReceived) {
     PTRACE(1, "SIP\tFailed to receive ACK!");
     ackRetry.Stop();
-    releaseMethod = ReleaseWithBYE;
-    Release(EndedByTemporaryFailure);
+    ackReceived = true;
+    if (GetPhase() < ReleasingPhase) {
+      releaseMethod = ReleaseWithBYE;
+      Release(EndedByTemporaryFailure);
+    }
   }
 }
 
