@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22169 $
+ * $Revision: 22589 $
  * $Author: rjongbloed $
- * $Date: 2009-03-09 02:18:57 +0000 (Mon, 09 Mar 2009) $
+ * $Date: 2009-05-12 10:18:55 +0000 (Tue, 12 May 2009) $
  */
 
 #include <ptlib.h>
@@ -1051,8 +1051,10 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
   PCaselessString str = GetUserAgent();
   if (str.IsEmpty()) {
     str = GetString("Server");
-    if (str.IsEmpty())
+    if (str.IsEmpty()) {
+      PTRACE(4, "SIP\tNo User-Agent or Server fields, Product Info unknown.");
       return; // Have nothing, change nothing
+    }
   }
 
   // This is not strictly correct according to he BNF, but we cheat
@@ -1066,6 +1068,7 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
     info.name = str;
     info.vendor = info.version = PString::Empty();
     info.manufacturerCode = info.t35Extension = info.t35CountryCode = 0;
+    PTRACE(4, "SIP\tProduct Info: name=\"" << str << '"');
     return;
   }
 
@@ -1077,6 +1080,10 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
   info.version = str(endFirstToken+1, endSecondToken);
   info.vendor = GetOrganization();
   info.comments = str.Mid(endSecondToken+1).Trim();
+  PTRACE(4, "SIP\tProduct Info: name=\"" << info.name << "\","
+                           " version=\"" << info.version << "\","
+                            " vendor=\"" << info.vendor << "\","
+                          " comments=\"" << info.comments << '"');
 }
 
 
@@ -1808,7 +1815,7 @@ void SIP_PDU::Construct(Methods meth,
   PINDEX pos = localPartyName.Find('@');
   if (pos != P_MAX_INDEX)
     localPartyName = localPartyName.Left(pos);
-  SIPURL contact = endpoint.GetContactURL(transport, localPartyName, SIPURL(connection.GetRemotePartyAddress()).GetHostName());
+  SIPURL contact = endpoint.GetContactURL(transport, SIPURL(localPartyName, connection.GetRemotePartyAddress(), endpoint.GetDefaultSignalPort()));
   contact.Sanitise(meth != Method_INVITE ? SIPURL::ContactURI : SIPURL::RouteURI);
   mime.SetContact(contact);
 
@@ -2706,13 +2713,11 @@ void SIPTransaction::OnRetry(PTimer &, INT)
 {
   PSafeLockReadWrite lock(*this);
 
-  if (!lock.IsLocked() || (state != Trying && state != Cancelling))
+  if (!lock.IsLocked() || state > Cancelling || (state == Proceeding && method == Method_INVITE))
     return;
 
   retry++;
 
-  PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " timeout, making retry " << retry);
-
   if (retry >= endpoint.GetMaxRetries()) {
     SetTerminated(Terminated_RetriesExceeded);
     return;
@@ -2727,11 +2732,16 @@ void SIPTransaction::OnRetry(PTimer &, INT)
       return;
   }
 
-  PTimeInterval timeout = retryTimeoutMin*(1<<retry);
-  if (timeout > retryTimeoutMax)
+  if (state > Trying)
     retryTimer = retryTimeoutMax;
-  else
+  else {
+    PTimeInterval timeout = retryTimeoutMin*(1<<retry);
+    if (timeout > retryTimeoutMax)
+      timeout = retryTimeoutMax;
     retryTimer = timeout;
+  }
+
+  PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " timeout, making retry " << retry << ", timeout " << retryTimer);
 }
 
 
